﻿/*
 *    Copyright (C) 2017 Sobey Inc.
 *
 *    This program is free software: you can redistribute it and/or  modify
 *    it under the terms of the GNU Affero General Public License, version 3,
 *    as published by the Free Software Foundation.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Affero General Public License for more details.
 *
 *    You should have received a copy of the GNU Affero General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
// stdafx.cpp : 只包括标准包含文件的源文件
// SobeyMemBaseLib.pch 将作为预编译头
// stdafx.obj 将包含预编译类型信息

#include "stdafx.h"
#include <math.h>
#ifdef WIN32
#include "md5.h"
#endif
#define PRECISION 0.00001

CCriticalSectionPtr::CCriticalSectionPtr(LPCRITICAL_SECTION lp)
{
	EnterCriticalSection(lp);
	m_lpcs = lp;
}

CCriticalSectionPtr::~CCriticalSectionPtr(void)
{
	LeaveCriticalSection(m_lpcs);
}

CExecutePlanCriticalPtr::CExecutePlanCriticalPtr(LPCRITICAL_SECTION lp, LPEXECUTEPLANBSON lpExecutePlan)
{
	EnterCriticalSection(lp);
	m_lpcs = lp;
	
	m_lpExecutePlan = lpExecutePlan;
	if(m_lpExecutePlan != NULL)
	{
		m_lpCritStack	= &lpExecutePlan->m_pCritStack[lpExecutePlan->m_nCritPos];
		*m_lpCritStack	= (long long)lp;
		lpExecutePlan->m_nCritPos++;
	}
}

CExecutePlanCriticalPtr::~CExecutePlanCriticalPtr(void)
{
	if(m_lpcs != NULL)
	{
		LeaveCriticalSection(m_lpcs);
	}

	if(m_lpExecutePlan != NULL)
	{
		*m_lpCritStack = 0;
		m_lpExecutePlan->m_nCritPos--;
	}
}

typedef struct stu_VernoxError_Def
{
	int			nErrorNO;					//错误码
	char		lpszDescription[128];		//错误描述
}VERNOXERROR_DEF;

VERNOXERROR_DEF g_pVernoxErrorDef[] = 
{
	//错误码			错误描述
	MF_OK,	"正确",
	MF_FAILED,	"执行失败",
	MF_INNER_EXPCETION_FAILED,	"异常失败",
	MF_INNER_ALLOCMEM_FAILED,	"分配内存失败",
	MF_INNER_POINTER_NULL,	"指针为空错误",
	MF_INNER_OPENMEMFILE_FAILED,	"打开内存文件失败",
	MF_INNER_MAPMEMFILE_FAILED,	"映射内存文件地址失败",
	MF_INNER_FILENO_ERROR,	"文件编号校验错误",
	MF_INNER_BLOCKNO_ERROR,	"数据块编号错误，找不到对应数据",
	MF_INNER_INNERNO_NOTEXIST,	"找不到对应数据",
	MF_INNER_INNERNO_DATAERROR,	"在块内通过块号定位的数据不正确",
	MF_INNER_ALLOWBLOCK_NOENOUGHSPACE,	"没有可用空间",
	MF_INNER_FILENO_NOTEXIST,	"引用的文件编号不存在",
	MF_INNER_OBJECTID_NOTEXIST,	"引用的对象ID不存在",
	MF_INNER_INDEXID_NOTEXIST,	"引用的索引ID不存在",
	MF_INNER_ITEMDATASIZE_ERROR,	"发现块头中记录的m_nBlockDataStructSize，与预期的不符",
	MF_INNER_INSTANCE_UNINITALIZE,	"实例未初始化",
	MF_INNER_MEMSIZE_ERROR,	"非法内存大小",
	MF_INNER_SYS_OBJECTMAP_DATAERROR,	"OBJECTMAP记录的OBJECTID不正确",
	MF_INNER_SYS_BLOCKSIZE_ERROR,	"块头记录的m_nBlockSize不正确，目前系统文件的所有块的大小都固定为256KB",
	MF_INNER_SYS_TRACE_INIT_ERROR,	"日志初始化失败",
	MF_INNER_SYS_TRACE_EVENT_FAILED,	"日志事件初始化失败",
	MF_INNER_SYS_UNINITIALIZE_FAILED,	"系统没有初始化",
	MF_INNER_SYS_INVALID_PARAMETER_ERROR,	"参数错误或者叫非法参数",
	MF_INNER_SYS_MEMDBFILENAME_INVALID,	"给定的内存文件名非法",
	MF_INNER_SYS_SOURCEOCCUPIED_INVALID,	"资源被占用",
	MF_INNER_SYS_SOURCERELEASE_ERROR,	"资源释放出错",
	MF_INNER_SYS_FULLOBJECT_ERROR,	"文件中对象已满",
	MF_INNER_SYS_RELEASELOCK_ERROR,	"释放锁失败",
	MF_INNER_SYS_OPENSHMEM_ERROR,	"打开共享内存失败",
	MF_INNER_SYS_INITIALIZECOM_INVALID,	"初始化COM组件错误",
	MF_INNER_SYS_CREATEXMLDOM_INVALID,	"创建XMLDOM失败",
	MF_INNER_SYS_CREATEXMLROOT_INVALID,	"创建XML根节点失败",
	MF_INNER_SYS_LOADXML_ERROR,	"加载XML失败",
	MF_INNER_SYS_NOMATCHNODE_ERROR,	"未找到指定xml结点",
	MF_INNER_SYS_NULLNODE_ERROR,	"结点为空",
	MF_INNER_SYS_TRANSACTIONLOGIC_EXIST_FAILED,	"事务字符串已经存在导致失败",
	MF_SYS_DBFILE_FILENAME_ERROR,	"添加或另存内存数据文件时，指定的新文件已经存在",
	MF_SYS_DBFILE_MEMFILENAME_ERROR,	"添加内存数据文件时，指定的内存文件名已经存在",
	MF_SYS_DBFILE_SERVICESTORAGE_NOTRESPONSE,	"添加内存数据文件时，请求服务执行无响应",
	MF_SYS_DBFILE_NOTEXIST,	"指定的内存数据文件不存在",
	MF_SYS_DBFILE_SERVICE_FILENO_ERROR,	"VernoxStorage检查到添加的文件编号重复错误",
	MF_SYS_DBFILE_SERVICE_FILEPATH_ERROR,	"VernoxStorage检查到添加的文件路径重复错误",
	MF_SYS_DBFILE_SERVICE_MEMFILE_ERROR,	"VernoxStorage检查到添加的内存文件名重复错误",
	MF_SYS_DBFILE_SERVICE_NEWFILE_FAILED,	"VernoxStorage保存新文件重复",
	MF_SYS_DBFILE_SERVICE_NOTEXIST,	"VernoxStorage没找到对应文件",
	MF_SYS_DBFILE_SERVICE_CHANGEFILE_FAILED,	"VernoxStorage改变文件路径失败",
	MF_SYS_DBFILE_FILENO_REPEATERROR,	"发现数据中出现了重复的文件编号，整个系统无法启动",
	MF_SYS_DBFILE_FILENO_ERROR,	"添加的文件编号重复错误",
	MF_SYS_DBFILE_FILEEXIST,	"添加或另存内存数据文件时，指定的新文件已经存在",
	MF_SYS_DBFILE_MEMFILEEXIST,	"添加内存数据文件时，指定的内存文件名已经存在",
	MF_SYS_DBFILE_NO_FILESIZE_ERROR,	"添加内存数据文件时，未指定文件大小",
	MF_SYS_DBFILE_NO_FILEPATH_ERROR,	"添加内存数据文件时，未指定文件路径",
	MF_SYS_DBFILE_NO_MEMFILE_ERROR,	"添加内存数据文件时，未指定内存文件名",
	MF_SYS_DBFILE_NO_FILETYPE_ERROR,	"添加内存数据文件时，文件类型错误",
	MF_SYS_DBFILE_NOSUPPORT_FILETYPE_FAILED,	"初始化内存文件实例时，遇到不支持的文件类型",
	MF_SYS_SEQUENCE_ADDNAMEEXIST,	"序列添加时，重名失败",
	MF_SYS_SEQUENCE_NOTEXIST,	"序列不存在",
	MF_SYS_SEQUENCE_MAXVALUE,	"序列值已经达到最大值",
	MF_SYS_SEQUENCE_NAME_REPEATERROR,	"发现序列中出现重复名字的序列，整个系统无法启动",
	MF_SYS_OBJECT_REPEATERROR,	"发现数据中出现了重复的对象编号，整个系统无法启动",
	MF_SYS_OBJECT_FIELDORDERERROR,	"发现对象数据中的字段序号不正确",
	MF_SYS_OBJECT_NOTEXIST,	"通过对象名找对象信息不存在",
	MF_SYS_OBJECT_ADDNAMEEXIST,	"数据对象添加时，重名失败",
	MF_SYS_OBJECT_NOFIELD,	"数据对象添加时没有列数据",
	MF_SYS_OBJECT_MANYFIELD,	"数据对象添加时列数据太多",
	MF_SYS_OBJECT_DATAFILE_NOTEXIST,	"数据对象添加时给定的数据文件名不存在",
	MF_SYS_OBJECT_BINDEXFILE_NOTEXIST,	"数据对象添加时给定的B树索引文件名不存在",
	MF_SYS_OBJECT_KVINDEXFILE_NOTEXIST,	"数据对象添加时给定的KV索引文件名不存在",
	MF_SYS_OBJECT_INDEXTYPE_NOTSUPPORT,	"数据索引类型不支持",
	MF_SYS_OBJECT_DELETENOTEXIST,	"删除数据对象添加时，没找到指定对象",
	MF_SYS_OBJECT_ADDFIELDNAME_EXIST,	"添加的对象字段出现重复",
	MF_SYS_OBJECT_FIELDNO_ERROR,	"数据对象添加时，有索引存在的情况下，字段序列不正确",
	MF_SYS_OBJECT_DELETEFIELDNO_NOTEXIST,	"删除字段时，字段名不存在",
	MF_SYS_OBJECT_DELETEINDEX_NOTEXIST,	"删除字段时，索引名不存在",
	MF_SYS_OBJECT_ADDINDEX_EXIST,	"添加索引时，索引名重复",
	MF_SYS_OBJECT_ADDINDEX_FILETYPE_ERROR,	"添加索引时，给定的文件类型和索引类型不符合",
	MF_SYS_OBJECT_ADDINDEX_FIELDNAME_ERROR,	"添加索引时，给定的索引字段找不到",
	MF_SYS_OBJECT_INDEX_INVALID_FIELDTYPE_ERROR,	"转换索引类型时遇到错误字段类型",
	MF_SYS_OBJECT_INDEX_INVALID_FILETYPE_ERROR,	"转换索引类型时遇到传入的文件类型为系统文件或者数据文件",
	MF_SYS_OBJECT_INDEX_NOSUPPORT_FILETYPE_ERROR,	"转换索引类型时遇到传入的文件类型不支持",
	MF_SYS_OBJECT_GRAPHFIELD_ERROR,	"图对象字段错误",
	MF_SYS_OBJECT_ALTERGRAPH_ERROR,	"图对象表结构不允许修改",
	MF_SYS_OBJECT_IMPLEMENTCLASS_ERROR, "对象实现类出错",
	MF_SYS_OBJECT_IMPLEMENTCLASS_LOAD, "系统正在载入文件，暂时禁用非系统对象插件",
	MF_SYS_OBJECT_ADDINDEX_MULTIPRIMRAYKEY_ERROR, "添加索引时，存在多个主键",
	MF_SYS_USER_NOTEXIST,	"用户信息不存在",
	MF_SYS_USER_ADDNAMEEXIST,	"添加用户时发现用户名已存在",
	MF_SYS_USER_REPEATERROR,	"发现数据中出现了重复的用户编号，整个系统无法启动",
	MF_SYS_AUTHORITY_NOTEXIST,	"用户信息不存在",
	MF_PARSECMD_INVALID_START_ERROR,	"解析命令时遇到非法开始字符错误",
	MF_PARSECMD_INVALID_CHAR_ERROR,	"解析命令时遇到非法字符错误",
	MF_PARSECMD_INVALID_COMMAND_ERROR,	"解析命令时遇到非法命令错误",
	MF_PARSECMD_INVALID_NULL_ERROR,	"解析命令时发现命令为空错误",
	MF_PARSECMD_INVALID_CREATE_SYNTAX_ERROR,	"解析CREATE命令时出现语法错误",
	MF_PARSECMD_INVALID_SELECT_SYNTAX_ERROR,	"解析SELECT命令时出现语法错误",
	MF_PARSECMD_INVALID_INSERT_SYNTAX_ERROR,	"解析INSERT命令时出现语法错误",
	MF_PARSECMD_INVALID_UPDATE_SYNTAX_ERROR,	"解析UPDATE命令时出现语法错误",
	MF_PARSECMD_INVALID_DELETE_SYNTAX_ERROR,	"解析DELETE命令时出现语法错误",
	MF_PARSECMD_INVALID_ALTER_SYNTAX_ERROR,	"解析ALTER命令时出现语法错误",
	MF_PARSECMD_OBJECTNAME_SYNTAX_ERROR,	"SQL语句中的期待对象名时，语法上有错误",
	MF_PARSECMD_FIELDNAME_SYNTAX_ERROR,	"SQL语句中的期待字段时，语法上有错误",
	MF_PARSECMD_INVALID_NUMBER_ERROR,	"SQL语句中的期待数字数据时，语法上有错误",
	MF_PARSECMD_INVALID_STRING_ERROR,	"SQL语句中的期待字符串数据时，语法上有错误",
	MF_PARSECMD_INVALID_OPERATOR_ERROR,	"SQL语句中的期待运算符时，语法上有错误",
	MF_PARSECMD_INVALID_EXPRESSION_ERROR,	"SQL语句中的期待表达式时，语法上有错误",
	MF_PARSECMD_INVALID_DATACOMPATIBLE_ERROR,	"SQL语句中的期待数据时，类型兼容上有错误",
	MF_PARSECMD_INVALID_FUNCTION_ERROR,	"SQL语句中的期待函数时，函数出错",
	MF_PARSECMD_INVALID_USER_ERROR,	"解析SQL语句时发现用户错误",
	MF_PARSECMD_INVALID_AUTHORITY_ERROR,	"解析SQL语句时发现权限错误",
	MF_PARSECMD_EXECUTEPALN_LACKSPACE_ERROR,	"解析SQL语句时发现执行计划空间不足",
	MF_PARSECMD_EXECUTEPALN_INVALID_SIZE_ERROR,	"解析SQL语句时发现执行计划大小有误",
	MF_PARSECMD_EMPTY_STRING,	"解析SQL语句发现是空字符串",
	MF_PARSECREATE_OBJECT_EXIT_ERROR,	"解析创建对象命令时，发现对象已存在",
	MF_PARSESELECT_INVALID_NULLOBJECT_ERROR,	"解析查询命令时发现表名为空错误",
	MF_PARSESELECT_INVALID_MULTIOBJECT_ERROR,	"解析查询命令时发现有多个表名错误",
	MF_PARSESELECT_INVALID_OBJECT_ERROR,	"解析查询命令时发现表名错误",
	MF_PARSESELECT_INVALID_FIELD_ERROR,	"解析查询命令时发现非法列名",
	MF_PARSESELECT_INVALID_QUERYINDEX_ERROR,	"解析查询命令时发现非法查询索引 ",
	MF_PARSESELECT_INVALID_FIELDRANGE_ERROR,	"解析查询命令时发现字段范围异常：如 SNO>20 AND SNO<10",
	MF_PARSESELECT_INVALID_COMPATIBLETYPE_ERROR,	"解析查询命令时发现数据兼容性异常",
	MF_PARSESELECT_INVALID_WHERECONDITION_ERROR,	"解析查询命令是发现查询条件错误",
	MF_PARSESELECT_INVALID_DATATYPE_ERROR,	"解析查询命令是发现数据类型错误",
	MF_PARSESELECT_INVALID_CONNECTION_ERROR,	"解析查询命令是发现CONNECTIONT条件错误",
	MF_PARSESELECT_INVALID_DRECITON_ERROR,	"解析查询命令是发现CONNECTIONT方向错误",
	MF_PARSESELECT_INVALID_SEQUENCE_ERROR,	"解析查询命令是发现序列错误",
	MF_PARSESELECT_INVALID_PAGING_ERROR,	"解析查询命令是发现分页错误",
	MF_PARSESELECT_INVALID_OBJECTTYPE_ERROR,	"解析查询命令对象类型不匹配",
	MF_PARSESELECT_INVALID_ROWNUMEXPRESSION_ERROR,	"非法ROWNUM表达式",
	MF_PARSEUPDATE_INVALID_QUERYINDEX_ERROR,	"解析更新命名时发现非法索引 ",
	MF_PARSEUPDATE_INVALID_FIELD_ERROR,	"解析更新命名时发现非法字段名",
	MF_PARSEUPDATE_INVALID_EXPRESSION_ERROR,	"解析更新命名时发现非法表达式",
	MF_PARSEUPDATE_INVALID_OBJECTTYPE_ERROR,	"解析更新命令对象类型不匹配",
	MF_PARSEUPDATE_INVALID_OBJECTNAME_ERROR,	"解析更新命令对象名不正确",
	MF_PARSEDELETE_INVALID_QUERYINDEX_ERROR,	"解析删除命令时发现非法删除索引 ",
	MF_PARSEDELETE_INVALID_OBJECT_ERROR,	"解析删除命令对发现表名错误",
	MF_PARSEDELETE_INVALID_OBJECTTYPE_ERROR,	"解析删除命令对象类型不匹配",
	MF_PARSEDELETE_INVALID_CONDITION_ERROR,	"解析删除命令对象类型不匹配",
	MF_PARSEALTER_INVALID_OBJECTTYPE_ERROR,	"解析修改命令对象类型不匹配",
	MF_PARSEISNERT_INVALID_FIELD_ERROR,	"解析插入命令时发现非法列名",
	MF_PARSEISNERT_INVALID_COMPATIBLETYPE_ERROR,	"解析插入命令时发现数据兼容类型错误",
	MF_PARSEISNERT_INVALID_NULLPOINTER_ERROR,	"解析插入命令时空指针错误",
	MF_PARSEISNERT_INVALID_OBJECTTYPE_ERROR,	"解析插入命令对象类型不匹配",
	MF_PARSEISNERT_INVALID_RELATIONNODECOND_ERROR,	"解析插入命令对象关系的节点条件错误",
	MF_PARSEINSERT_INVALID_OBJECT_ERROR,	"解析插入命令时发现表名错误",
	MF_CREATELOGIC_INVALID_NULLQUEUE_ERROR,	"创建执行逻辑时发现后缀表达式栈队列空错误",
	MF_CREATELOGIC_INVALID_QUEUE_ERROR,	"创建执行逻辑时发现后缀表达式队列错误",
	MF_CREATELOGIC_INVALID_LOGICSTACK_ERROR,	"创建执行逻辑时发现逻辑达式栈错误",
	MF_CREATEEXPRESSION_INVALID_NULLQUEUE_ERROR,	"创建表达式时发现后缀表达式队列为空错误",
	MF_CREATEEXPRESSION_INVALID_QUEUE_ERROR,	"创建表达式时发现后缀表达式队列错误",
	MF_CREATEEXPRESSION_INVALID_EXPRESSIONSTACK_ERROR,	"创建表达式时发现表达式栈错误",
	MF_EXECUTEQUERY_INVALID_INDEXCONDITION_ERROR,	"执行查询语句时发现索引条件错误",
	MF_EXECUTEQUERY_INVALID_DATA_ERROR,	"执行查询语句时发现数据错误",
	MF_EXECUTEQUERY_INVALID_NULLFIELD_ERROR,	"执行查询语句时发现空字段",
	MF_EXECUTEQUERY_INVALID_NULLDATA_ERROR,	"执行查询语句时发现没有满足条件的字段",
	MF_EXECUTEQUERY_INVALID_OPERATOR_ERROR,	"执行查询语句时发现运算符错误",
	MF_EXECUTEQUERY_INVALID_ANALYSISBUFFER_ERROR,	"执行查询语句时反序列化Buffer错误",
	MF_EXECUTEQUERY_INVALID_NULLPOINT_ERROR,	"执行查询语句时空指针异常",
	MF_EXECUTEUPDATE_INVALID_ANALYSISBUFFER_ERROR,	"执行修改语句时反序列化Buffer",
	MF_EXECUTEUPDATE_INVALID_NULLDATA_ERROR,	"执行修改语句时发现没有满足条件的字段",
	MF_EXECUTEUPDATE_INVALID_NULLFIELD_ERROR,	"执行修改语句时发现不允许为空的字段为空错误",
	MF_EXECUTEUPDATE_INVALID_FIELD_ERROR,	"执行修改语句时发现字段非法",
	MF_EXECUTEUPDATE_INVALID_CHARLEN_ERROR,	"执行修改语句时发现定长字符串长度异常",
	MF_EXECUTEDELETE_INVALID_ANALYSISBUFFER_ERROR,	"执行删除语句时反序列化Buffer",
	MF_EXECUTEDELETE_INVALID_NULLDATA_ERROR,	"执行删除语句时发现没有满足条件的字段",
	MF_EXECUTEINSERT_INVALID_NULLFIELD_ERROR,	"执行插入语句时发现不允许为空的字段为空错误",
	MF_EXECUTEINSERT_INVALID_APPENDDATA_ERROR,	"执行插入语句时序列化数据失败",
	MF_EXECUTEINSERT_INVALID_EXPRESSION_ERROR,	"执行插入语句时表达式错误",
	MF_EXECUTERECORDSETUPDATE_INVALID_NULLEXEPLAN_ERROR,	"执行结果集更新时，发现执行计划为空",
	MF_PARSECMD_INVALID_DROP_SYNTAX_ERROR,	"删除语句语法错误",
	MF_PARSECMD_DROP_LACK_KEYWORD_ERROR,	"删除语句缺少关键词错误",
	MF_PARSECMD_CREATE_LACK_WITH_ERROR,	"创建语句缺少With错误",
	MF_PARSECMD_CREATE_LACK_BY_ERROR,	"创建语句缺少By错误",
	MF_PARSECMD_CREATE_LACK_BRACE_ERROR,	"创建语句缺少括号错误",
	MF_PARSECMD_CREATE_FIELD_NOTEXISTS_ERROR,	"创建语句中的列不存在错误",
	MF_PARSECMD_CREATE_LACK_KEYWORD_ERROR,	"创建语句缺少关键词错误",
	MF_PARSECMD_CREATE_INDEXNAME_EXIST_ERROR,	"创建语句索引名称重复，错误",
	MF_PARSECMD_CREATE_LACK_PRIMARYINDEX_ERROR,	"创建语句主键缺少索引错误",
	MF_PARSECMD_CREATE_HBINDEX_FIELDTYPE_NOTQUEAL,	"创建语句中HB树索引的主键字段与父亲字段的数据类型不相同",
	MF_PARSECMD_ALTER_LACK_BRACE_ERROR,	"修改语句缺少括号错误",
	MF_PARSECMD_ALTER_LACK_KEYWORD_ERROR,	"修改语句缺少关键词错误",
	MF_PARSECMD_ALTER_SET_SYSTEM_INIT_ERROR,	"修改系统参数，初始化未初始化完成，无法获取系统参数",
	MF_PARSECMD_ALTER_SET_SYSTEM_PARAMETERNAME_ERROR,	"修改系统参数，参数值不合法",
	MF_PARSECMD_ALTER_INDEX_TYPE_ERROR,	"修改索引时，发现给定的数据文件类型",
	MF_PARSECMD_INVALID_GRANT_SYNTAX_ERROR,	"授权语句语法错误",
	MF_PARSECMD_INVALID_REVOKE_SYNTAX_ERROR,	"授权语句语法错误",
	MF_INTERFACE_NOT_OPEN,	"未打开数据库链接",
	MF_INTERFACE_SOCKET_CREATE_FAILED,	"连接服务器时，创建Socket失败",
	MF_INTERFACE_SOCKET_CONNECT_FAILED,	"连接服务器失败",
	MF_INTERFACE_SENDDATA_FAILED,	"发送数据失败",
	MF_INTERFACE_RECEIVEDATA_TIMEOUT,	"接收数据超时",
	MF_INTERFACE_RECEIVEDATA_CONNECTRESET,	"接收数据链接重置",
	MF_INTERFACE_RECEIVEDATA_FAILED,	"接收数据失败",
	MF_INTERFACE_RECEIVEDATA_DATAERROR,	"接收数据错误",
	MF_INTERFACE_RECEIVEDATA_TOKENERROR,	"通讯令牌错误",
	MF_INTERFACE_SERVER_PARSE_FAILED,	"服务名解析失败",
	MF_INTERFACE_SERVER_PARSE_ERROR,	"未找到传入的服务名",
	MF_INTERFACE_CLIENT_LOAD_FAILED,	"加载VernoxClient.dll库失败",
	MF_INTERFACE_CLIENT_FUNC_FAILED,	"定位VernoxClient.dll库函数失败",
	//非系统级错误码定义，从1000开始			
	//文件级异常1000~1049			
	MF_MEMORYFILE_ALLOCBLOCK_NOENOUGHMEMORY_ERROR,	"分配空块时，空间不足错误",
	MF_MEMORYFILE_ALLOCBLOCK_INVALIDDATAFLAG_ERROR,	"分配空块时，块标识错误",
	MF_MEMORYFILE_ALLOCBLOCK_NULLPOINTBLOCK_ERROR,	"分配空块时，未找到指定块",
	MF_MEMORYFILE_ALLOCBLOCK_INVALDRESOURCETYPE_ERROR,	"分配空块时，块来源错误",
	MF_MEMORYFILE_ALLOCBLOCK_LOCKNEWBLOCK_ERROR,	"分配空块时，锁定新块失败",
	MF_MEMORYFILE_ALLOCNEWDATA_NOENOUGHMEMORY_ERROR,	"分配新数据时，空间不足",
	MF_MEMORYFILE_MERGEMEMORY_BLOCKPOINTER_ERROR,	"整理文件时，块指针错误",
	MF_MEMORYFILE_INSERTMAPSTRUCT_ERROR,	"插入块映射表错误",
	MF_MEMORYFILE_MOVEMAPSTRUCT_ERROR,	"插入块映射表错误",
	MF_MEMORYFILE_DATA_TIMESTAMP_ERROR				,"数据时间戳错误",
	MF_MEMORYFILE_CONVERTDATAID_INVALIDDATAID_ERROR,	"将DataID转换成对象指针时，DataID错误",
	MF_MEMORYFILE_FINDMAPSTRUCT_INVALIDLIST_ERROR,	"在某链表(满块链表或空块链表)中查找某一结点时，链表起始地址错误",
	MF_MEMORYFILE_FINDMAPSTRUCT_NOBLOCKMATCH_ERROR,	"在某链表(满块链表或空块链表)中查找某一结点时，没有相应块匹配错误",
	MF_MEMORYFILE_GETFIELDDATA_INVALIDFIELDNO_ERROR,	"根据字段编号获取字段值时，字段编号错误",
	MF_MEMORYFILE_GETSTRINGDATA_INVALIDVARDATA_ERROR,	"获取字符串字段时，变长数据错误",
	MF_MEMORYFILE_GETVARDATA_INVALIDVARDATA_ERROR,	"获取变长数据时，变长数据错误",
	MF_MEMORYFILE_GETINSERTDATAID_NEWBLOCKLOCK_ERROR,	"绑定插入记录时，发现新分配的块已被上锁",
	MF_MEMORYFILE_GETFILEOBJECT_NOOBJECTMATHC_ERROR,	"获取对象信息时，无对象匹配",
	MF_MEMORYFILE_GETBLOCKPTR_BLOCKNO_ERROR,	"获取对象信息时，无对象匹配",
	MF_MEMORYFILE_INSERTDATA_INVALIDMEMORYMAP_ERROR,	"插入数据时，映射表错误",
	MF_MEMORYFILE_INSERTDATA_INVALIDOBJECTID_ERROR,	"插入数据时，ObjectID错误",
	MF_MEMORYFILE_INSERTDATA_INVALIDDATAID_ERROR,	"插入数据时，DataID错误",
	MF_MEMORYFILE_INSERTDATA_NOENOUGHMEMORY_ERROR,	"插入数据时，空间不足错误",
	MF_MEMORYFILE_READDATA_INVALIDDATAID_ERROR,	"读取数据时，DataID错误",
	MF_MEMORYFILE_READDATA_NULLDATA_ERROR,	"读取数据时，数据已被删除",
	MF_MEMORYFILE_UPDATEDATA_INVALIDDATAID_ERROR,	"更新数据时，DataID错误",
	MF_MEMORYFILE_UPDATEROLLBACK_BUFFERSIZE_ERROR,	"更新回滚时，空间长度错误",
	MF_MEMORYFILE_DELETEDATA_INVALIDDATAID_ERROR,	"删除数据时，DataID错误",

	//B树索引相关异常1050~1099			
	MF_BTREEINDEX_INVALIDROOTNO_ERROR,	"根节点编号错误",
	MF_BTREEINDEX_INVALIDNODENO_ERROR,	"节点编号错误",
	MF_BTREEINDEX_INVALIDNODETYPE_ERROR,	"节点类型错误",
	MF_BTREEINDEX_INVALIDINXEXID_ERROR,	"IndexID错误",
	MF_BTREEINDEX_INVALIDINDEXTYPE_ERROR,	"索引类型错误",
	MF_BTREEINDEX_ALLOCBLOCK_NOENOUGHMEMORY_ERROR,	"没有足够的空间分配内存错误",
	MF_BTREEINDEX_CONVERTNODENO_INVALIDNODENO_ERROR,	"将块号转换为指针时，块号异常",
	MF_BTREEINDEX_FINDMAPSTRUCT_INVALIDLIST_ERROR,	"在某链表(满块链表或空块链表)中查找某一结点时，链表起始地址错误",
	MF_BTREEINDEX_FINDMAPSTRUCT_NOBLOCKMATCH_ERROR,	"在某链表(满块链表或空块链表)中查找某一结点时，没有节点匹配",
	MF_BTREEINDEX_TREESEARCH_NULLPOINTER_ERROR,	"遍历B树时，空指针错误",
	MF_BTREEINDEX_TREESEARCH_NULLDATA,	"遍历B树时，没有找到合适数据",
	MF_BTREEINDEX_BINARYSEARCH_NOKEYMATH,	"遍历B树时，没有匹配的关键字",
	MF_BTREEINDEX_BINARYSEARCH_NOKEYEQUAL,	"遍历B树时，没有相同的关键字",
	MF_BTREEINDEX_UNIQUEKEY_EXIST,	"遍历B树时，唯一关键字存在",
	MF_BTREEINDEX_DELETESTRING_IDNOTMATCH,	"删除B树索引字符串关键字时DATAID不匹配",
	MF_BTREEINDEX_GETMULTIKEYINFO_ERROR,	"获取字符串符合索引的关键字信息失败",
	MF_BTREEINDEX_DELETEKEY_KEYPOSERROR,	"删除B树索引字符串关键字时，关键字位置错误",
	MF_BTREEINDEX_DELETEINDEX_NODEERROR,	"删除B树索引字符串关键字时,节点错误",
	MF_BTREEINDEX_MERGENODE_ERROR,   		"B树索引合并错误",
	MF_BTREEINDEX_RECYCLEBUFFERSIZE_ERROR,  "回收Buffer大小错误",
	//KV索引相关异常1100~1149			
	MF_KVINDEX_INVALIDROOTNO_ERROR,	"根节点编号错误",
	MF_KVINDEX_INVALIDBLOCKNO_ERROR,	"节点编号错误",
	MF_KVINDEX_INVALIDINDEXID_ERROR,	"IndexID错误",
	MF_KVINDEX_INVALINDEXOFFSET_ERROR,	"索引偏移错误",
	MF_KVINDEX_INVALIDINDEXTYPE_ERROR,	"索引类型错误",
	MF_KVINDEX_MULTIKEY_ERROR,	"关键字值重复",
	MF_KVINDEX_ALLOCBLOCK_NOENOUGHMEMORY_ERROR,	"没有足够的空间分配内存错误",
	MF_KVINDEX_PARENT_NOTFOUND_ERROR,	"二叉树父亲结点未找到",

	//回滚区异常1150~1159			
	MF_MEMANALYSIS_NULLPOINTER_ERROR,	"空指针异常",
	MF_ROLLBACKBUFFER_GETDATA_BLOCKNUM_ERROR,	"获取回滚区数据轮数失效错误",
	MF_ROLLBACKBUFFER_GETDATA_OFFSET_ERROR,	"获取回滚区数据偏移量错误",
	MF_ROLLBACKBUFFER_GETDATA_TIMESTAMP_ERROR,	"获取回滚区数据时间错误",

	//分页异常1160~1169			
	MF_PAGING_ERROR,	"分页异常",
	MF_PAGING_LACKPAGEBUFFER_ERROR,	"页面Buffer不足",
	MF_PAGING_INVALIDBUFFERNO_ERROR,	"非法缓存编号",
	MF_PAGING_INVALIDPAGENO_ERROR,	"非法页编号",
	MF_PAGING_PAGETIMEOUT_ERROR,	"页面失效(被覆盖)",

	//HB树索引相关异常1170~1179			
	MF_HBINDEX_DELETEINDEX_EXSITCHILD_ERROR,	"HB树结点存在孩子结点，删除失败",
	MF_HBINDEX_UPDATEINDEX_EXSITCHILD_ERROR,	"HB树结点存在孩子结点，更新失败",
	MF_HBINDEX_UPDATEINDEX_INVALIDPID_ERROR,	"父亲结点错误，删除失败",

	//记录Buffer相关异常1200~1250			
	MF_ROWDATABUFFER_NULL_POINTER,	"空指针异常",
	MF_ROWDATABUFFER_NULL_DATABUFFER,	"空数据异常",
	MF_ROWDATABUFFER_INVALID_FIELDTYPE,	"非法列类型",
	MF_ROWDATABUFFER_INVALID_BUFFERLEN,	"非法BUFFER长度",

	//Check的相关异常			
	MF_CHECK_GETFIELDVALUE_ERROR,	"从记录中获取字段值异常",

	//公共异常1500以后			
	MF_COMMON_INVALID_DATA,	"非法数据",
	MF_COMMON_INVALID_DATATYPE,	"非法数据类型",
	MF_COMMON_INVALID_FUNTYPE,	"非法函数类型",
	MF_COMMON_INVALID_OPERATOR,	"非法运算符",
	MF_COMMON_INVALID_DATEFORMAT,	"非法日期格式",
	MF_COMMON_INVALID_DATE,	"非法日期",
	MF_COMMON_INVALID_CMDTYPE,	"非法命令类型",
	MF_COMMON_INVALID_FIELD,	"非法列",
	MF_COMMON_INVALID_OBJECTID,	"错误对象ID",
	MF_COMMON_INVALID_COMPARECONDITION,	"比较条件错误",
	MF_COMMON_INVALID_STRINGLEN,	"字符串长度错误",
	MF_COMMON_INVALID_NULLPRIMARYKEY,	"空主键异常",
	MF_COMMON_INVALID_INDEXTYPE,	"非法索引类型",
	MF_COMMON_INVALID_NONEPRIMARYKEY,	"缺少主键",
	MF_COMMON_MAX_DATANUM,	"记录数已达上限",
	MF_COMMON_MAX_OBJECT,	"对象数已达上限",
	MF_COMMON_MAX_INDEX,	"索引数已达上限",
	MF_COMMON_INVALID_STEPTYPE,	"步骤类型错误",
	MF_COMMON_INVALID_OBJECTTYPE,	"对象类型错误",
	MF_COMMON_RECORD_NOTEXIT,	"记录不存在",
	MF_COMMON_INVALID_FIELDTYPE,	"非法列",
	MF_COMMON_INVALID_EXECUTEBLOCKNUM,	"操作所涉及的块数量超过限制",
	MF_COMMON_INVALID_DATABASE,	"非法数据库",
	MF_COMMON_INVALID_FIELDNO,	"非法列编号",
	MF_COMMON_INVALID_BUFFERSIZE,	"非法缓存大小",
	MF_COMMON_FILE_NOTEXIT,	"文件不存在",
	//创建执行计划异常			
	MF_CREATEPLAN_GETRECORDLEN_FAILED,	"创建执行计划时获取记录长度失败",
	MF_CREATEPLAN_INVALIDBUFFERNO_FAILED,	"创建执行计划时缓存编号错误",

	//登录异常信息			
	MF_USERLOGIN_INVALIDUSERNAME_FAILED,	"用户名不存在",
	MF_USERLOGIN_INVALIDPASSWORD_FAILED,	"密码不正确",
	MF_USERLOGIN_DISABLE_USER,				"用户被禁用",

	//闪回异常信息			
	MF_FLASHBACK_TIMEOUT,	"调用闪回超时",
	MF_FLASHBACK_INVALID_MAP_SIZE,	"调用闪回时检测到映射表大小异常",

	//验证数据库GUID的异常			
	MF_SYS_DATABASE_GUID_UNMATCH,	"验证数据库GUID失败，读到的日志文件与数据库不匹配",

	//获取记录的错误			
	MF_GETRECORD_INVALID_RECORDSECTIONARRAY,	"获取记录时，记录片段数组错误",

	//索引优化器异常			
	MF_INDEXOPTIMIZER_CONDITIONOFFSET_ERROR,	"索引优化器中，查询条件偏移异常",
	MF_INDEXOPTIMIZER_EXPRESSIONOFFSET_ERROR,	"索引优化器中，表达式偏移异常",
	MF_INDEXOPTIMIZER_LOGICOPERATOR_ERROR,	"索引优化器中，逻辑运算符错误",
	MF_INDEXOPTIMIZER_COMPAREOPERATOR_ERROR,	"索引优化器中，比较运算符错误",
	MF_INDEXOPTIMIZER_MATHEXPTYPE_ERROR,	"索引优化器中，算数运算符类型错误",
	MF_INDEXOPTIMIZER_SUBINDEXPLANSTACK_ERROR,	"索引优化器中，子索引计划栈错误",
	MF_INDEXOPTIMIZER_INDEXNUM_ERROR,	"索引优化器中，索引数量错误",
	MF_INDEXOPTIMIZER_INDEXTYPE_ERROR,	"索引优化器中，索引类型错误",
	MF_INDEXOPTIMIZER_INDEXCONDITION_ERROR,	"索引优化器中，索引条件错误",
	MF_INDEXOPTIMIZER_GETOPTIMALINDEX_ERROR,	"索引优化器中，获取最佳索引错误",
	//返回值			
	MF_CLUSTER_REDO_FAILED,	"集群出现错误",
	MF_CLUSTER_REDO_LACK_OF_BUFFER,	"集群缓存太小，需要重分配",
	MF_CLUSTER_NEW_REDO,	"有新的日志文件/打开新的日志文件成功",
	MF_CLUSTER_NO_NEW_REDO,	"没有新的日志文件",
	MF_CLUSTER_READ_REDO_SUCCEED,	"读取一条执行计划成功",
	MF_CLUSTER_CURRENT_REDO_FINISH,	"已读取到当前文件末尾",
	MF_CLUSTER_NO_REDO,	"没有日志文件",

	//重作日志恢复错误
	MF_REDO_INIT_FAILED,	"初始化重作日志管理结构体失败",
	MF_REDO_TIMESTAMP_LESSTHAN_ZERO,	"重作日志恢复时，输入的时间戳小于零",
	MF_REDO_TIMESTAMP_LESSTHAN_FIRSTONE,	"重作日志恢复时，输入的时间小于第一条时间戳",
	MF_REDO_REDOID_LESSEQUAL_ZERO,	"重作日志恢复时，日志文件编号小于等于零",
	MF_REDO_READ_SIZE_NOT_EQUAL,	"重作日志恢复时，读取数据的大小不对",
	MF_REDO_CHECK_GUID_FAIL,		"重作日志恢复时，校验数据库guid失败",
	MF_REDO_TIMESTAMP_NOT_FOUND,	"重作日志恢复时，找不到指定时间戳对应的记录",
	MF_REDO_NO_REDO_FILE,			"重作日志恢复时，没有日志文件",

	//更新结果错误			
	MF_UPDATERECORDSET_FIELDNOTEXIT_ERROR,	"更新结果集字段值不存在",
	MF_UPDATERECORDSET_FIELDCANNOTMODIFY_ERROR,	"更新结果集字段不允许修改",
	MF_UPDATERECORDSET_RECORDSETCANNOTMODIFY_ERROR,	"结果集不允许修改",
	MF_UPDATERECORDSET_INVALID_NULLFIELD_ERROR,	"结果集时非空字段为空错误",
	MF_UPDATERECORDSET_INVALID_FIELD_ERROR,	"结果集时字段异常",
};


typedef struct stu_WVernoxError_Def
{
	int			nErrorNO;					//错误码
	wchar_t		lpwszDescription[128];		//错误描述
}WVERNOXERROR_DEF;

WVERNOXERROR_DEF g_pwVernoxErrorDef[] = 
{
	//错误码			错误描述
//错误码			错误描述
	MF_OK,	L"正确",
	MF_FAILED,	L"执行失败",
	MF_INNER_EXPCETION_FAILED,	L"异常失败",
	MF_INNER_ALLOCMEM_FAILED,	L"分配内存失败",
	MF_INNER_POINTER_NULL,	L"指针为空错误",
	MF_INNER_OPENMEMFILE_FAILED,	L"打开内存文件失败",
	MF_INNER_MAPMEMFILE_FAILED,	L"映射内存文件地址失败",
	MF_INNER_FILENO_ERROR,	L"文件编号校验错误",
	MF_INNER_BLOCKNO_ERROR,	L"数据块编号错误，找不到对应数据",
	MF_INNER_INNERNO_NOTEXIST,	L"找不到对应数据",
	MF_INNER_INNERNO_DATAERROR,	L"在块内通过块号定位的数据不正确",
	MF_INNER_ALLOWBLOCK_NOENOUGHSPACE,	L"没有可用空间",
	MF_INNER_FILENO_NOTEXIST,	L"引用的文件编号不存在",
	MF_INNER_OBJECTID_NOTEXIST,	L"引用的对象ID不存在",
	MF_INNER_INDEXID_NOTEXIST,	L"引用的索引ID不存在",
	MF_INNER_ITEMDATASIZE_ERROR,	L"发现块头中记录的m_nBlockDataStructSize，与预期的不符",
	MF_INNER_INSTANCE_UNINITALIZE,	L"实例未初始化",
	MF_INNER_MEMSIZE_ERROR,	L"非法内存大小",
	MF_INNER_SYS_OBJECTMAP_DATAERROR,	L"OBJECTMAP记录的OBJECTID不正确",
	MF_INNER_SYS_BLOCKSIZE_ERROR,	L"块头记录的m_nBlockSize不正确，目前系统文件的所有块的大小都固定为256KB",
	MF_INNER_SYS_TRACE_INIT_ERROR,	L"日志初始化失败",
	MF_INNER_SYS_TRACE_EVENT_FAILED,	L"日志事件初始化失败",
	MF_INNER_SYS_UNINITIALIZE_FAILED,	L"系统没有初始化",
	MF_INNER_SYS_INVALID_PARAMETER_ERROR,	L"参数错误或者叫非法参数",
	MF_INNER_SYS_MEMDBFILENAME_INVALID,	L"给定的内存文件名非法",
	MF_INNER_SYS_SOURCEOCCUPIED_INVALID,	L"资源被占用",
	MF_INNER_SYS_SOURCERELEASE_ERROR,	L"资源释放出错",
	MF_INNER_SYS_FULLOBJECT_ERROR,	L"文件中对象已满",
	MF_INNER_SYS_RELEASELOCK_ERROR,	L"释放锁失败",
	MF_INNER_SYS_OPENSHMEM_ERROR,	L"打开共享内存失败",
	MF_INNER_SYS_INITIALIZECOM_INVALID,	L"初始化COM组件错误",
	MF_INNER_SYS_CREATEXMLDOM_INVALID,	L"创建XMLDOM失败",
	MF_INNER_SYS_CREATEXMLROOT_INVALID,	L"创建XML根节点失败",
	MF_INNER_SYS_LOADXML_ERROR,	L"加载XML失败",
	MF_INNER_SYS_NOMATCHNODE_ERROR,	L"未找到指定xml结点",
	MF_INNER_SYS_NULLNODE_ERROR,	L"结点为空",
	MF_INNER_SYS_TRANSACTIONLOGIC_EXIST_FAILED,	L"事务字符串已经存在导致失败",
	MF_SYS_DBFILE_FILENAME_ERROR,	L"添加或另存内存数据文件时，指定的新文件已经存在",
	MF_SYS_DBFILE_MEMFILENAME_ERROR,	L"添加内存数据文件时，指定的内存文件名已经存在",
	MF_SYS_DBFILE_SERVICESTORAGE_NOTRESPONSE,	L"添加内存数据文件时，请求服务执行无响应",
	MF_SYS_DBFILE_NOTEXIST,	L"指定的内存数据文件不存在",
	MF_SYS_DBFILE_SERVICE_FILENO_ERROR,	L"VernoxStorage检查到添加的文件编号重复错误",
	MF_SYS_DBFILE_SERVICE_FILEPATH_ERROR,	L"VernoxStorage检查到添加的文件路径重复错误",
	MF_SYS_DBFILE_SERVICE_MEMFILE_ERROR,	L"VernoxStorage检查到添加的内存文件名重复错误",
	MF_SYS_DBFILE_SERVICE_NEWFILE_FAILED,	L"VernoxStorage保存新文件重复",
	MF_SYS_DBFILE_SERVICE_NOTEXIST,	L"VernoxStorage没找到对应文件",
	MF_SYS_DBFILE_SERVICE_CHANGEFILE_FAILED,	L"VernoxStorage改变文件路径失败",
	MF_SYS_DBFILE_FILENO_REPEATERROR,	L"发现数据中出现了重复的文件编号，整个系统无法启动",
	MF_SYS_DBFILE_FILENO_ERROR,	L"添加的文件编号重复错误",
	MF_SYS_DBFILE_FILEEXIST,	L"添加或另存内存数据文件时，指定的新文件已经存在",
	MF_SYS_DBFILE_MEMFILEEXIST,	L"添加内存数据文件时，指定的内存文件名已经存在",
	MF_SYS_DBFILE_NO_FILESIZE_ERROR,	L"添加内存数据文件时，未指定文件大小",
	MF_SYS_DBFILE_NO_FILEPATH_ERROR,	L"添加内存数据文件时，未指定文件路径",
	MF_SYS_DBFILE_NO_MEMFILE_ERROR,	L"添加内存数据文件时，未指定内存文件名",
	MF_SYS_DBFILE_NO_FILETYPE_ERROR,	L"添加内存数据文件时，文件类型错误",
	MF_SYS_DBFILE_NOSUPPORT_FILETYPE_FAILED,	L"初始化内存文件实例时，遇到不支持的文件类型",
	MF_SYS_SEQUENCE_ADDNAMEEXIST,	L"序列添加时，重名失败",
	MF_SYS_SEQUENCE_NOTEXIST,	L"序列不存在",
	MF_SYS_SEQUENCE_MAXVALUE,	L"序列值已经达到最大值",
	MF_SYS_SEQUENCE_NAME_REPEATERROR,	L"发现序列中出现重复名字的序列，整个系统无法启动",
	MF_SYS_OBJECT_REPEATERROR,	L"发现数据中出现了重复的对象编号，整个系统无法启动",
	MF_SYS_OBJECT_FIELDORDERERROR,	L"发现对象数据中的字段序号不正确",
	MF_SYS_OBJECT_NOTEXIST,	L"通过对象名找对象信息不存在",
	MF_SYS_OBJECT_ADDNAMEEXIST,	L"数据对象添加时，重名失败",
	MF_SYS_OBJECT_NOFIELD,	L"数据对象添加时没有列数据",
	MF_SYS_OBJECT_MANYFIELD,	L"数据对象添加时列数据太多",
	MF_SYS_OBJECT_DATAFILE_NOTEXIST,	L"数据对象添加时给定的数据文件名不存在",
	MF_SYS_OBJECT_BINDEXFILE_NOTEXIST,	L"数据对象添加时给定的B树索引文件名不存在",
	MF_SYS_OBJECT_KVINDEXFILE_NOTEXIST,	L"数据对象添加时给定的KV索引文件名不存在",
	MF_SYS_OBJECT_INDEXTYPE_NOTSUPPORT,	L"数据索引类型不支持",
	MF_SYS_OBJECT_DELETENOTEXIST,	L"删除数据对象添加时，没找到指定对象",
	MF_SYS_OBJECT_ADDFIELDNAME_EXIST,	L"添加的对象字段出现重复",
	MF_SYS_OBJECT_FIELDNO_ERROR,	L"数据对象添加时，有索引存在的情况下，字段序列不正确",
	MF_SYS_OBJECT_DELETEFIELDNO_NOTEXIST,	L"删除字段时，字段名不存在",
	MF_SYS_OBJECT_DELETEINDEX_NOTEXIST,	L"删除字段时，索引名不存在",
	MF_SYS_OBJECT_ADDINDEX_EXIST,	L"添加索引时，索引名重复",
	MF_SYS_OBJECT_ADDINDEX_FILETYPE_ERROR,	L"添加索引时，给定的文件类型和索引类型不符合",
	MF_SYS_OBJECT_ADDINDEX_FIELDNAME_ERROR,	L"添加索引时，给定的索引字段找不到",
	MF_SYS_OBJECT_INDEX_INVALID_FIELDTYPE_ERROR,	L"转换索引类型时遇到错误字段类型",
	MF_SYS_OBJECT_INDEX_INVALID_FILETYPE_ERROR,	L"转换索引类型时遇到传入的文件类型为系统文件或者数据文件",
	MF_SYS_OBJECT_INDEX_NOSUPPORT_FILETYPE_ERROR,	L"转换索引类型时遇到传入的文件类型不支持",
	MF_SYS_OBJECT_GRAPHFIELD_ERROR,	L"图对象字段错误",
	MF_SYS_OBJECT_ALTERGRAPH_ERROR,	L"图对象表结构不允许修改",
	MF_SYS_OBJECT_IMPLEMENTCLASS_ERROR, L"对象实现类出错",
	MF_SYS_OBJECT_IMPLEMENTCLASS_LOAD, L"系统正在载入文件，暂时禁用非系统对象插件",
	MF_SYS_OBJECT_ADDINDEX_MULTIPRIMRAYKEY_ERROR, L"添加索引时，存在多个主键",
	MF_SYS_USER_NOTEXIST,	L"用户信息不存在",
	MF_SYS_USER_ADDNAMEEXIST,	L"添加用户时发现用户名已存在",
	MF_SYS_USER_REPEATERROR,	L"发现数据中出现了重复的用户编号，整个系统无法启动",
	MF_SYS_AUTHORITY_NOTEXIST,	L"用户信息不存在",
	MF_PARSECMD_INVALID_START_ERROR,	L"解析命令时遇到非法开始字符错误",
	MF_PARSECMD_INVALID_CHAR_ERROR,	L"解析命令时遇到非法字符错误",
	MF_PARSECMD_INVALID_COMMAND_ERROR,	L"解析命令时遇到非法命令错误",
	MF_PARSECMD_INVALID_NULL_ERROR,	L"解析命令时发现命令为空错误",
	MF_PARSECMD_INVALID_CREATE_SYNTAX_ERROR,	L"解析CREATE命令时出现语法错误",
	MF_PARSECMD_INVALID_SELECT_SYNTAX_ERROR,	L"解析SELECT命令时出现语法错误",
	MF_PARSECMD_INVALID_INSERT_SYNTAX_ERROR,	L"解析INSERT命令时出现语法错误",
	MF_PARSECMD_INVALID_UPDATE_SYNTAX_ERROR,	L"解析UPDATE命令时出现语法错误",
	MF_PARSECMD_INVALID_DELETE_SYNTAX_ERROR,	L"解析DELETE命令时出现语法错误",
	MF_PARSECMD_INVALID_ALTER_SYNTAX_ERROR,	L"解析ALTER命令时出现语法错误",
	MF_PARSECMD_OBJECTNAME_SYNTAX_ERROR,	L"SQL语句中的期待对象名时，语法上有错误",
	MF_PARSECMD_FIELDNAME_SYNTAX_ERROR,	L"SQL语句中的期待字段时，语法上有错误",
	MF_PARSECMD_INVALID_NUMBER_ERROR,	L"SQL语句中的期待数字数据时，语法上有错误",
	MF_PARSECMD_INVALID_STRING_ERROR,	L"SQL语句中的期待字符串数据时，语法上有错误",
	MF_PARSECMD_INVALID_OPERATOR_ERROR,	L"SQL语句中的期待运算符时，语法上有错误",
	MF_PARSECMD_INVALID_EXPRESSION_ERROR,	L"SQL语句中的期待表达式时，语法上有错误",
	MF_PARSECMD_INVALID_DATACOMPATIBLE_ERROR,	L"SQL语句中的期待数据时，类型兼容上有错误",
	MF_PARSECMD_INVALID_FUNCTION_ERROR,	L"SQL语句中的期待函数时，函数出错",
	MF_PARSECMD_INVALID_USER_ERROR,	L"解析SQL语句时发现用户错误",
	MF_PARSECMD_INVALID_AUTHORITY_ERROR,	L"解析SQL语句时发现权限错误",
	MF_PARSECMD_EXECUTEPALN_LACKSPACE_ERROR,	L"解析SQL语句时发现执行计划空间不足",
	MF_PARSECMD_EXECUTEPALN_INVALID_SIZE_ERROR,	L"解析SQL语句时发现执行计划大小有误",
	MF_PARSECMD_EMPTY_STRING,	L"解析SQL语句发现是空字符串",
	MF_PARSECREATE_OBJECT_EXIT_ERROR,	L"解析创建对象命令时，发现对象已存在",
	MF_PARSESELECT_INVALID_NULLOBJECT_ERROR,	L"解析查询命令时发现表名为空错误",
	MF_PARSESELECT_INVALID_MULTIOBJECT_ERROR,	L"解析查询命令时发现有多个表名错误",
	MF_PARSESELECT_INVALID_OBJECT_ERROR,	L"解析查询命令时发现表名错误",
	MF_PARSESELECT_INVALID_FIELD_ERROR,	L"解析查询命令时发现非法列名",
	MF_PARSESELECT_INVALID_QUERYINDEX_ERROR,	L"解析查询命令时发现非法查询索引 ",
	MF_PARSESELECT_INVALID_FIELDRANGE_ERROR,	L"解析查询命令时发现字段范围异常：如 SNO>20 AND SNO<10",
	MF_PARSESELECT_INVALID_COMPATIBLETYPE_ERROR,	L"解析查询命令时发现数据兼容性异常",
	MF_PARSESELECT_INVALID_WHERECONDITION_ERROR,	L"解析查询命令是发现查询条件错误",
	MF_PARSESELECT_INVALID_DATATYPE_ERROR,	L"解析查询命令是发现数据类型错误",
	MF_PARSESELECT_INVALID_CONNECTION_ERROR,	L"解析查询命令是发现CONNECTIONT条件错误",
	MF_PARSESELECT_INVALID_DRECITON_ERROR,	L"解析查询命令是发现CONNECTIONT方向错误",
	MF_PARSESELECT_INVALID_SEQUENCE_ERROR,	L"解析查询命令是发现序列错误",
	MF_PARSESELECT_INVALID_PAGING_ERROR,	L"解析查询命令是发现分页错误",
	MF_PARSESELECT_INVALID_OBJECTTYPE_ERROR,	L"解析查询命令对象类型不匹配",
	MF_PARSESELECT_INVALID_ROWNUMEXPRESSION_ERROR,	L"非法ROWNUM表达式",
	MF_PARSEUPDATE_INVALID_QUERYINDEX_ERROR,	L"解析更新命名时发现非法索引 ",
	MF_PARSEUPDATE_INVALID_FIELD_ERROR,	L"解析更新命名时发现非法字段名",
	MF_PARSEUPDATE_INVALID_EXPRESSION_ERROR,	L"解析更新命名时发现非法表达式",
	MF_PARSEUPDATE_INVALID_OBJECTTYPE_ERROR,	L"解析更新命令对象类型不匹配",
	MF_PARSEUPDATE_INVALID_OBJECTNAME_ERROR,	L"解析更新命令对象名不正确",
	MF_PARSEDELETE_INVALID_QUERYINDEX_ERROR,	L"解析删除命令时发现非法删除索引 ",
	MF_PARSEDELETE_INVALID_OBJECT_ERROR,	L"解析删除命令对发现表名错误",
	MF_PARSEDELETE_INVALID_OBJECTTYPE_ERROR,	L"解析删除命令对象类型不匹配",
	MF_PARSEDELETE_INVALID_CONDITION_ERROR,	L"解析删除命令对象类型不匹配",
	MF_PARSEALTER_INVALID_OBJECTTYPE_ERROR,	L"解析修改命令对象类型不匹配",
	MF_PARSEISNERT_INVALID_FIELD_ERROR,	L"解析插入命令时发现非法列名",
	MF_PARSEISNERT_INVALID_COMPATIBLETYPE_ERROR,	L"解析插入命令时发现数据兼容类型错误",
	MF_PARSEISNERT_INVALID_NULLPOINTER_ERROR,	L"解析插入命令时空指针错误",
	MF_PARSEISNERT_INVALID_OBJECTTYPE_ERROR,	L"解析插入命令对象类型不匹配",
	MF_PARSEISNERT_INVALID_RELATIONNODECOND_ERROR,	L"解析插入命令对象关系的节点条件错误",
	MF_PARSEINSERT_INVALID_OBJECT_ERROR,	L"解析插入命令时发现表名错误",
	MF_CREATELOGIC_INVALID_NULLQUEUE_ERROR,	L"创建执行逻辑时发现后缀表达式栈队列空错误",
	MF_CREATELOGIC_INVALID_QUEUE_ERROR,	L"创建执行逻辑时发现后缀表达式队列错误",
	MF_CREATELOGIC_INVALID_LOGICSTACK_ERROR,	L"创建执行逻辑时发现逻辑达式栈错误",
	MF_CREATEEXPRESSION_INVALID_NULLQUEUE_ERROR,	L"创建表达式时发现后缀表达式队列为空错误",
	MF_CREATEEXPRESSION_INVALID_QUEUE_ERROR,	L"创建表达式时发现后缀表达式队列错误",
	MF_CREATEEXPRESSION_INVALID_EXPRESSIONSTACK_ERROR,	L"创建表达式时发现表达式栈错误",
	MF_EXECUTEQUERY_INVALID_INDEXCONDITION_ERROR,	L"执行查询语句时发现索引条件错误",
	MF_EXECUTEQUERY_INVALID_DATA_ERROR,	L"执行查询语句时发现数据错误",
	MF_EXECUTEQUERY_INVALID_NULLFIELD_ERROR,	L"执行查询语句时发现空字段",
	MF_EXECUTEQUERY_INVALID_NULLDATA_ERROR,	L"执行查询语句时发现没有满足条件的字段",
	MF_EXECUTEQUERY_INVALID_OPERATOR_ERROR,	L"执行查询语句时发现运算符错误",
	MF_EXECUTEQUERY_INVALID_ANALYSISBUFFER_ERROR,	L"执行查询语句时反序列化Buffer错误",
	MF_EXECUTEQUERY_INVALID_NULLPOINT_ERROR,	L"执行查询语句时空指针异常",
	MF_EXECUTEUPDATE_INVALID_ANALYSISBUFFER_ERROR,	L"执行修改语句时反序列化Buffer",
	MF_EXECUTEUPDATE_INVALID_NULLDATA_ERROR,	L"执行修改语句时发现没有满足条件的字段",
	MF_EXECUTEUPDATE_INVALID_NULLFIELD_ERROR,	L"执行修改语句时发现不允许为空的字段为空错误",
	MF_EXECUTEUPDATE_INVALID_FIELD_ERROR,	L"执行修改语句时发现字段非法",
	MF_EXECUTEUPDATE_INVALID_CHARLEN_ERROR,	L"执行修改语句时发现定长字符串长度异常",
	MF_EXECUTEDELETE_INVALID_ANALYSISBUFFER_ERROR,	L"执行删除语句时反序列化Buffer",
	MF_EXECUTEDELETE_INVALID_NULLDATA_ERROR,	L"执行删除语句时发现没有满足条件的字段",
	MF_EXECUTEINSERT_INVALID_NULLFIELD_ERROR,	L"执行插入语句时发现不允许为空的字段为空错误",
	MF_EXECUTEINSERT_INVALID_APPENDDATA_ERROR,	L"执行插入语句时序列化数据失败",
	MF_EXECUTEINSERT_INVALID_EXPRESSION_ERROR,	L"执行插入语句时表达式错误",
	MF_EXECUTERECORDSETUPDATE_INVALID_NULLEXEPLAN_ERROR,	L"执行结果集更新时，发现执行计划为空",
	MF_PARSECMD_INVALID_DROP_SYNTAX_ERROR,	L"删除语句语法错误",
	MF_PARSECMD_DROP_LACK_KEYWORD_ERROR,	L"删除语句缺少关键词错误",
	MF_PARSECMD_CREATE_LACK_WITH_ERROR,	L"创建语句缺少With错误",
	MF_PARSECMD_CREATE_LACK_BY_ERROR,	L"创建语句缺少By错误",
	MF_PARSECMD_CREATE_LACK_BRACE_ERROR,	L"创建语句缺少括号错误",
	MF_PARSECMD_CREATE_FIELD_NOTEXISTS_ERROR,	L"创建语句中的列不存在错误",
	MF_PARSECMD_CREATE_LACK_KEYWORD_ERROR,	L"创建语句缺少关键词错误",
	MF_PARSECMD_CREATE_INDEXNAME_EXIST_ERROR,	L"创建语句索引名称重复，错误",
	MF_PARSECMD_CREATE_LACK_PRIMARYINDEX_ERROR,	L"创建语句主键缺少索引错误",
	MF_PARSECMD_CREATE_HBINDEX_FIELDTYPE_NOTQUEAL,	L"创建语句中HB树索引的主键字段与父亲字段的数据类型不相同",
	MF_PARSECMD_ALTER_LACK_BRACE_ERROR,	L"修改语句缺少括号错误",
	MF_PARSECMD_ALTER_LACK_KEYWORD_ERROR,	L"修改语句缺少关键词错误",
	MF_PARSECMD_ALTER_SET_SYSTEM_INIT_ERROR,	L"修改系统参数，初始化未初始化完成，无法获取系统参数",
	MF_PARSECMD_ALTER_SET_SYSTEM_PARAMETERNAME_ERROR,	L"修改系统参数，参数值不合法",
	MF_PARSECMD_ALTER_INDEX_TYPE_ERROR,	L"修改索引时，发现给定的数据文件类型",
	MF_PARSECMD_INVALID_GRANT_SYNTAX_ERROR,	L"授权语句语法错误",
	MF_PARSECMD_INVALID_REVOKE_SYNTAX_ERROR,	L"授权语句语法错误",
	MF_INTERFACE_NOT_OPEN,	L"未打开数据库链接",
	MF_INTERFACE_SOCKET_CREATE_FAILED,	L"连接服务器时，创建Socket失败",
	MF_INTERFACE_SOCKET_CONNECT_FAILED,	L"连接服务器失败",
	MF_INTERFACE_SENDDATA_FAILED,	L"发送数据失败",
	MF_INTERFACE_RECEIVEDATA_TIMEOUT,	L"接收数据超时",
	MF_INTERFACE_RECEIVEDATA_CONNECTRESET,	L"接收数据链接重置",
	MF_INTERFACE_RECEIVEDATA_FAILED,	L"接收数据失败",
	MF_INTERFACE_RECEIVEDATA_DATAERROR,	L"接收数据错误",
	MF_INTERFACE_RECEIVEDATA_TOKENERROR,	L"通讯令牌错误",
	MF_INTERFACE_SERVER_PARSE_FAILED,	L"服务名解析失败",
	MF_INTERFACE_SERVER_PARSE_ERROR,	L"未找到传入的服务名",
	MF_INTERFACE_CLIENT_LOAD_FAILED,	L"加载VernoxClient.dll库失败",
	MF_INTERFACE_CLIENT_FUNC_FAILED,	L"定位VernoxClient.dll库函数失败",
	//非系统级错误码定义，从1000开始			
	//文件级异常1000~1049			
	MF_MEMORYFILE_ALLOCBLOCK_NOENOUGHMEMORY_ERROR,	L"分配空块时，空间不足错误",
	MF_MEMORYFILE_ALLOCBLOCK_INVALIDDATAFLAG_ERROR,	L"分配空块时，块标识错误",
	MF_MEMORYFILE_ALLOCBLOCK_NULLPOINTBLOCK_ERROR,	L"分配空块时，未找到指定块",
	MF_MEMORYFILE_ALLOCBLOCK_INVALDRESOURCETYPE_ERROR,	L"分配空块时，块来源错误",
	MF_MEMORYFILE_ALLOCBLOCK_LOCKNEWBLOCK_ERROR,	L"分配空块时，锁定新块失败",
	MF_MEMORYFILE_ALLOCNEWDATA_NOENOUGHMEMORY_ERROR,	L"分配新数据时，空间不足",
	MF_MEMORYFILE_MERGEMEMORY_BLOCKPOINTER_ERROR,	L"整理文件时，块指针错误",
	MF_MEMORYFILE_INSERTMAPSTRUCT_ERROR,	L"插入块映射表错误",
	MF_MEMORYFILE_MOVEMAPSTRUCT_ERROR,	L"插入块映射表错误",
	MF_MEMORYFILE_DATA_TIMESTAMP_ERROR				,L"数据时间戳错误",
	MF_MEMORYFILE_CONVERTDATAID_INVALIDDATAID_ERROR,	L"将DataID转换成对象指针时，DataID错误",
	MF_MEMORYFILE_FINDMAPSTRUCT_INVALIDLIST_ERROR,	L"在某链表(满块链表或空块链表)中查找某一结点时，链表起始地址错误",
	MF_MEMORYFILE_FINDMAPSTRUCT_NOBLOCKMATCH_ERROR,	L"在某链表(满块链表或空块链表)中查找某一结点时，没有相应块匹配错误",
	MF_MEMORYFILE_GETFIELDDATA_INVALIDFIELDNO_ERROR,	L"根据字段编号获取字段值时，字段编号错误",
	MF_MEMORYFILE_GETSTRINGDATA_INVALIDVARDATA_ERROR,	L"获取字符串字段时，变长数据错误",
	MF_MEMORYFILE_GETVARDATA_INVALIDVARDATA_ERROR,	L"获取变长数据时，变长数据错误",
	MF_MEMORYFILE_GETINSERTDATAID_NEWBLOCKLOCK_ERROR,	L"绑定插入记录时，发现新分配的块已被上锁",
	MF_MEMORYFILE_GETFILEOBJECT_NOOBJECTMATHC_ERROR,	L"获取对象信息时，无对象匹配",
	MF_MEMORYFILE_GETBLOCKPTR_BLOCKNO_ERROR,	L"获取对象信息时，无对象匹配",
	MF_MEMORYFILE_INSERTDATA_INVALIDMEMORYMAP_ERROR,	L"插入数据时，映射表错误",
	MF_MEMORYFILE_INSERTDATA_INVALIDOBJECTID_ERROR,	L"插入数据时，ObjectID错误",
	MF_MEMORYFILE_INSERTDATA_INVALIDDATAID_ERROR,	L"插入数据时，DataID错误",
	MF_MEMORYFILE_INSERTDATA_NOENOUGHMEMORY_ERROR,	L"插入数据时，空间不足错误",
	MF_MEMORYFILE_READDATA_INVALIDDATAID_ERROR,	L"读取数据时，DataID错误",
	MF_MEMORYFILE_READDATA_NULLDATA_ERROR,	L"读取数据时，数据已被删除",
	MF_MEMORYFILE_UPDATEDATA_INVALIDDATAID_ERROR,	L"更新数据时，DataID错误",
	MF_MEMORYFILE_UPDATEROLLBACK_BUFFERSIZE_ERROR,	L"更新回滚时，空间长度错误",
	MF_MEMORYFILE_DELETEDATA_INVALIDDATAID_ERROR,	L"删除数据时，DataID错误",

	//B树索引相关异常1050~1099			
	MF_BTREEINDEX_INVALIDROOTNO_ERROR,	L"根节点编号错误",
	MF_BTREEINDEX_INVALIDNODENO_ERROR,	L"节点编号错误",
	MF_BTREEINDEX_INVALIDNODETYPE_ERROR,	L"节点类型错误",
	MF_BTREEINDEX_INVALIDINXEXID_ERROR,	L"IndexID错误",
	MF_BTREEINDEX_INVALIDINDEXTYPE_ERROR,	L"索引类型错误",
	MF_BTREEINDEX_ALLOCBLOCK_NOENOUGHMEMORY_ERROR,	L"没有足够的空间分配内存错误",
	MF_BTREEINDEX_CONVERTNODENO_INVALIDNODENO_ERROR,	L"将块号转换为指针时，块号异常",
	MF_BTREEINDEX_FINDMAPSTRUCT_INVALIDLIST_ERROR,	L"在某链表(满块链表或空块链表)中查找某一结点时，链表起始地址错误",
	MF_BTREEINDEX_FINDMAPSTRUCT_NOBLOCKMATCH_ERROR,	L"在某链表(满块链表或空块链表)中查找某一结点时，没有节点匹配",
	MF_BTREEINDEX_TREESEARCH_NULLPOINTER_ERROR,	L"遍历B树时，空指针错误",
	MF_BTREEINDEX_TREESEARCH_NULLDATA,	L"遍历B树时，没有找到合适数据",
	MF_BTREEINDEX_BINARYSEARCH_NOKEYMATH,	L"遍历B树时，没有匹配的关键字",
	MF_BTREEINDEX_BINARYSEARCH_NOKEYEQUAL,	L"遍历B树时，没有相同的关键字",
	MF_BTREEINDEX_UNIQUEKEY_EXIST,	L"遍历B树时，唯一关键字存在",
	MF_BTREEINDEX_DELETESTRING_IDNOTMATCH,	L"删除B树索引字符串关键字时DATAID不匹配",
	MF_BTREEINDEX_GETMULTIKEYINFO_ERROR,	L"获取字符串符合索引的关键字信息失败",
	MF_BTREEINDEX_DELETEKEY_KEYPOSERROR,	L"删除B树索引字符串关键字时，关键字位置错误",
	MF_BTREEINDEX_DELETEINDEX_NODEERROR,	L"删除B树索引字符串关键字时,节点错误",
	MF_BTREEINDEX_MERGENODE_ERROR,   		L"B树索引合并错误",
	MF_BTREEINDEX_RECYCLEBUFFERSIZE_ERROR,  L"回收Buffer大小错误",
	//KV索引相关异常1100~1149			
	MF_KVINDEX_INVALIDROOTNO_ERROR,	L"根节点编号错误",
	MF_KVINDEX_INVALIDBLOCKNO_ERROR,	L"节点编号错误",
	MF_KVINDEX_INVALIDINDEXID_ERROR,	L"IndexID错误",
	MF_KVINDEX_INVALINDEXOFFSET_ERROR,	L"索引偏移错误",
	MF_KVINDEX_INVALIDINDEXTYPE_ERROR,	L"索引类型错误",
	MF_KVINDEX_MULTIKEY_ERROR,	L"关键字值重复",
	MF_KVINDEX_ALLOCBLOCK_NOENOUGHMEMORY_ERROR,	L"没有足够的空间分配内存错误",
	MF_KVINDEX_PARENT_NOTFOUND_ERROR,	L"二叉树父亲结点未找到",

	//回滚区异常1150~1159			
	MF_MEMANALYSIS_NULLPOINTER_ERROR,	L"空指针异常",
	MF_ROLLBACKBUFFER_GETDATA_BLOCKNUM_ERROR,	L"获取回滚区数据轮数失效错误",
	MF_ROLLBACKBUFFER_GETDATA_OFFSET_ERROR,	L"获取回滚区数据偏移量错误",
	MF_ROLLBACKBUFFER_GETDATA_TIMESTAMP_ERROR,	L"获取回滚区数据时间错误",

	//分页异常1160~1169			
	MF_PAGING_ERROR,	L"分页异常",
	MF_PAGING_LACKPAGEBUFFER_ERROR,	L"页面Buffer不足",
	MF_PAGING_INVALIDBUFFERNO_ERROR,	L"非法缓存编号",
	MF_PAGING_INVALIDPAGENO_ERROR,	L"非法页编号",
	MF_PAGING_PAGETIMEOUT_ERROR,	L"页面失效(被覆盖)",

	//HB树索引相关异常1170~1179			
	MF_HBINDEX_DELETEINDEX_EXSITCHILD_ERROR,	L"HB树结点存在孩子结点，删除失败",
	MF_HBINDEX_UPDATEINDEX_EXSITCHILD_ERROR,	L"HB树结点存在孩子结点，更新失败",
	MF_HBINDEX_UPDATEINDEX_INVALIDPID_ERROR,	L"父亲结点错误，删除失败",

	//记录Buffer相关异常1200~1250			
	MF_ROWDATABUFFER_NULL_POINTER,	L"空指针异常",
	MF_ROWDATABUFFER_NULL_DATABUFFER,	L"空数据异常",
	MF_ROWDATABUFFER_INVALID_FIELDTYPE,	L"非法列类型",
	MF_ROWDATABUFFER_INVALID_BUFFERLEN,	L"非法BUFFER长度",

	//Check的相关异常			
	MF_CHECK_GETFIELDVALUE_ERROR,	L"从记录中获取字段值异常",

	//公共异常1500以后			
	MF_COMMON_INVALID_DATA,	L"非法数据",
	MF_COMMON_INVALID_DATATYPE,	L"非法数据类型",
	MF_COMMON_INVALID_FUNTYPE,	L"非法函数类型",
	MF_COMMON_INVALID_OPERATOR,	L"非法运算符",
	MF_COMMON_INVALID_DATEFORMAT,	L"非法日期格式",
	MF_COMMON_INVALID_DATE,	L"非法日期",
	MF_COMMON_INVALID_CMDTYPE,	L"非法命令类型",
	MF_COMMON_INVALID_FIELD,	L"非法列",
	MF_COMMON_INVALID_OBJECTID,	L"错误对象ID",
	MF_COMMON_INVALID_COMPARECONDITION,	L"比较条件错误",
	MF_COMMON_INVALID_STRINGLEN,	L"字符串长度错误",
	MF_COMMON_INVALID_NULLPRIMARYKEY,	L"空主键异常",
	MF_COMMON_INVALID_INDEXTYPE,	L"非法索引类型",
	MF_COMMON_INVALID_NONEPRIMARYKEY,	L"缺少主键",
	MF_COMMON_MAX_DATANUM,	L"记录数已达上限",
	MF_COMMON_MAX_OBJECT,	L"对象数已达上限",
	MF_COMMON_MAX_INDEX,	L"索引数已达上限",
	MF_COMMON_INVALID_STEPTYPE,	L"步骤类型错误",
	MF_COMMON_INVALID_OBJECTTYPE,	L"对象类型错误",
	MF_COMMON_RECORD_NOTEXIT,	L"记录不存在",
	MF_COMMON_INVALID_FIELDTYPE,	L"非法列",
	MF_COMMON_INVALID_EXECUTEBLOCKNUM,	L"操作所涉及的块数量超过限制",
	MF_COMMON_INVALID_DATABASE,	L"非法数据库",
	MF_COMMON_INVALID_FIELDNO,	L"非法列编号",
	MF_COMMON_INVALID_BUFFERSIZE,	L"非法缓存大小",
	MF_COMMON_FILE_NOTEXIT,	L"文件不存在",
	//创建执行计划异常			
	MF_CREATEPLAN_GETRECORDLEN_FAILED,	L"创建执行计划时获取记录长度失败",
	MF_CREATEPLAN_INVALIDBUFFERNO_FAILED,	L"创建执行计划时缓存编号错误",

	//登录异常信息			
	MF_USERLOGIN_INVALIDUSERNAME_FAILED,	L"用户名不存在",
	MF_USERLOGIN_INVALIDPASSWORD_FAILED,	L"密码不正确",
	MF_USERLOGIN_DISABLE_USER,				L"用户被禁用",

	//闪回异常信息			
	MF_FLASHBACK_TIMEOUT,	L"调用闪回超时",
	MF_FLASHBACK_INVALID_MAP_SIZE,	L"调用闪回时检测到映射表大小异常",

	//验证数据库GUID的异常			
	MF_SYS_DATABASE_GUID_UNMATCH,	L"验证数据库GUID失败，读到的日志文件与数据库不匹配",

	//获取记录的错误			
	MF_GETRECORD_INVALID_RECORDSECTIONARRAY,	L"获取记录时，记录片段数组错误",

	//索引优化器异常			
	MF_INDEXOPTIMIZER_CONDITIONOFFSET_ERROR,	L"索引优化器中，查询条件偏移异常",
	MF_INDEXOPTIMIZER_EXPRESSIONOFFSET_ERROR,	L"索引优化器中，表达式偏移异常",
	MF_INDEXOPTIMIZER_LOGICOPERATOR_ERROR,	L"索引优化器中，逻辑运算符错误",
	MF_INDEXOPTIMIZER_COMPAREOPERATOR_ERROR,	L"索引优化器中，比较运算符错误",
	MF_INDEXOPTIMIZER_MATHEXPTYPE_ERROR,	L"索引优化器中，算数运算符类型错误",
	MF_INDEXOPTIMIZER_SUBINDEXPLANSTACK_ERROR,	L"索引优化器中，子索引计划栈错误",
	MF_INDEXOPTIMIZER_INDEXNUM_ERROR,	L"索引优化器中，索引数量错误",
	MF_INDEXOPTIMIZER_INDEXTYPE_ERROR,	L"索引优化器中，索引类型错误",
	MF_INDEXOPTIMIZER_INDEXCONDITION_ERROR,	L"索引优化器中，索引条件错误",
	MF_INDEXOPTIMIZER_GETOPTIMALINDEX_ERROR,	L"索引优化器中，获取最佳索引错误",
	//返回值			
	MF_CLUSTER_REDO_FAILED,	L"集群出现错误",
	MF_CLUSTER_REDO_LACK_OF_BUFFER,	L"集群缓存太小，需要重分配",
	MF_CLUSTER_NEW_REDO,	L"有新的日志文件/打开新的日志文件成功",
	MF_CLUSTER_NO_NEW_REDO,	L"没有新的日志文件",
	MF_CLUSTER_READ_REDO_SUCCEED,	L"读取一条执行计划成功",
	MF_CLUSTER_CURRENT_REDO_FINISH,	L"已读取到当前文件末尾",
	MF_CLUSTER_NO_REDO,	L"没有日志文件",

	//重作日志恢复错误
	MF_REDO_INIT_FAILED,	L"初始化重作日志管理结构体失败",
	MF_REDO_TIMESTAMP_LESSTHAN_ZERO,	L"重作日志恢复时，输入的时间戳小于零",
	MF_REDO_TIMESTAMP_LESSTHAN_FIRSTONE,	L"重作日志恢复时，输入的时间小于第一条时间戳",
	MF_REDO_REDOID_LESSEQUAL_ZERO,	L"重作日志恢复时，日志文件编号小于等于零",
	MF_REDO_READ_SIZE_NOT_EQUAL,	L"重作日志恢复时，读取数据的大小不对",
	MF_REDO_CHECK_GUID_FAIL,		L"重作日志恢复时，校验数据库guid失败",
	MF_REDO_TIMESTAMP_NOT_FOUND,	L"重作日志恢复时，找不到指定时间戳对应的记录",
	MF_REDO_NO_REDO_FILE,			L"重作日志恢复时，没有日志文件",

	//更新结果错误			
	MF_UPDATERECORDSET_FIELDNOTEXIT_ERROR,	L"更新结果集字段值不存在",
	MF_UPDATERECORDSET_FIELDCANNOTMODIFY_ERROR,	L"更新结果集字段不允许修改",
	MF_UPDATERECORDSET_RECORDSETCANNOTMODIFY_ERROR,	L"结果集不允许修改",
	MF_UPDATERECORDSET_INVALID_NULLFIELD_ERROR,	L"结果集时非空字段为空错误",
	MF_UPDATERECORDSET_INVALID_FIELD_ERROR,	L"结果集时字段异常",
};

#ifdef WIN32
char* ConvertToUTF8(const wchar_t * strData)
{
	// wcs -> utf8
	char * lpszData;
	if(strData == NULL)
		return NULL;
	int nLength = (int)wcslen(strData);
	if(nLength == 0)
		return NULL;
	int nLen = WideCharToMultiByte(CP_UTF8, NULL, strData,  nLength, NULL, NULL, NULL, NULL);
	lpszData = new char[nLen + 1];
	if(lpszData == NULL)
		return NULL;
	memset(lpszData,0,nLen + 1);
	WideCharToMultiByte(CP_UTF8, NULL, strData, nLength, lpszData, nLen, NULL, NULL);
	return lpszData;
}

char* ConvertToUTF8(const wchar_t * strData, int &nLen)
{
	// wcs -> utf8
	nLen = 0;
	int nLength;
	char * lpszData;
	if(strData == NULL)
		return NULL;
	nLength = (int)wcslen(strData);
	if(nLength == 0)
		return NULL;
	nLen = WideCharToMultiByte(CP_UTF8, NULL, strData,  nLength, NULL, NULL, NULL, NULL) + 1;
	lpszData = new char[nLen];
	if(lpszData == NULL)
		return NULL;
	memset(lpszData,0,nLen);
	WideCharToMultiByte(CP_UTF8, NULL, strData, nLength, lpszData, nLen, NULL, NULL);
	return lpszData;
}

char* ConvertBSTR2UTF8(const wchar_t * strData, int &nLen)
{
	// wcs -> utf8
	int nLength;
	char * lpszData;
	if(strData == NULL)
		return NULL;
	nLength = nLen;
	if(nLength == 0)
		return NULL;
	nLen = WideCharToMultiByte(CP_UTF8, NULL, strData,  nLength, NULL, NULL, NULL, NULL) + 1;
	lpszData = new char[nLen];
	if(lpszData == NULL)
		return NULL;
	memset(lpszData,0,nLen);
	WideCharToMultiByte(CP_UTF8, NULL, strData, nLength, lpszData, nLen, NULL, NULL);
	return lpszData;
}

wchar_t* ConvertWCHAR(const char * lpszData, int nLength)
{
	// utf8 -> wcs
	wchar_t *lpwsData;
	if(lpszData == NULL)
		return NULL;
	int nLen = MultiByteToWideChar(CP_UTF8, NULL, lpszData, nLength, NULL, NULL);
	lpwsData = new wchar_t[nLen + 1];
	if(lpwsData == NULL)
		return NULL;
	memset(lpwsData, 0, (nLen + 1)*sizeof(wchar_t));
	MultiByteToWideChar(CP_UTF8, NULL, lpszData, nLength, lpwsData, nLen);
	return lpwsData;
}

CComBSTR ConvertBSTR(const char * lpszData)
{
	// utf8 -> BSTR
	CComBSTR bstrData;
	wchar_t *lpwsData;
	if(lpszData == NULL)
		return bstrData;
	int nLength = (int)strlen(lpszData);
	if(nLength == 0)
		return bstrData;
	int nLen = MultiByteToWideChar(CP_UTF8, NULL, lpszData, nLength, NULL, NULL);
	lpwsData = new wchar_t[nLen + 1];
	if(lpwsData == NULL)
		return bstrData;
	memset(lpwsData, 0, (nLen + 1)*sizeof(wchar_t));
	MultiByteToWideChar(CP_UTF8, NULL, lpszData, nLength, lpwsData, nLen);
	bstrData = lpwsData;
	delete [] lpwsData;
	return bstrData;
}

std::string ConvertString(const wchar_t * strData)
{
	// utf8 -> string
	std::string sData;
	char * lpszData;
	if(strData == NULL)
		return sData;
	int nLength = (int)wcslen(strData);
	if(nLength == 0)
		return sData;
	int nLen = WideCharToMultiByte(CP_UTF8, NULL, strData,  nLength, NULL, NULL, NULL, NULL);
	lpszData = new char[nLen + 1];
	if(lpszData == NULL)
		return sData;
	memset(lpszData,0,nLen + 1);
	WideCharToMultiByte(CP_UTF8, NULL, strData, nLength, lpszData, nLen, NULL, NULL);
	
	sData = lpszData;
	delete [] lpszData;
	return sData;
}

char* UTF8ConvertAnsi(const char * lpszData)
{
	// utf8 -> Ansi
	char * lpData;
	wchar_t *lpwsData;
	int nLen, nAnsiLen;
	if(lpszData == NULL)
		return NULL;
	nLen = MultiByteToWideChar(CP_UTF8, NULL, lpszData, -1, NULL, NULL);
	lpwsData = new wchar_t[nLen + 1];
	if(lpwsData == NULL)
		return NULL;
	memset(lpwsData, 0, (nLen + 1)*sizeof(wchar_t));
	MultiByteToWideChar(CP_UTF8, NULL, lpszData, -1, lpwsData, nLen);
	nAnsiLen = WideCharToMultiByte(CP_ACP, NULL, lpwsData,  nLen, NULL, NULL, NULL, NULL);
	lpData = new char[nAnsiLen + 1];
	WideCharToMultiByte(CP_ACP, 0, lpwsData, nLen, lpData, nAnsiLen, NULL, NULL);
	delete [] lpwsData;
	return lpData;
}

string UTF8ConvertASC(const char * lpszData)
{
	// utf8 -> acs
	char * lpData;
	string strData;
	wchar_t *lpwsData;
	int nLen, nAnsiLen;
	if(lpszData == NULL)
		return strData;
	nLen = MultiByteToWideChar(CP_UTF8, NULL, lpszData, -1, NULL, NULL);
	lpwsData = new wchar_t[nLen + 1];
	if(lpwsData == NULL)
		return strData;
	memset(lpwsData, 0, (nLen + 1)*sizeof(wchar_t));
	MultiByteToWideChar(CP_UTF8, NULL, lpszData, -1, lpwsData, nLen);
	nAnsiLen = WideCharToMultiByte(CP_ACP, NULL, lpwsData,  nLen, NULL, NULL, NULL, NULL);
	lpData = new char[nAnsiLen + 1];
	WideCharToMultiByte(CP_ACP, 0, lpwsData, nLen, lpData, nAnsiLen, NULL, NULL);
	strData = lpData;
	
	delete [] lpwsData;
	delete [] lpData;
	return strData;
}

//将UTF8转换成Unicode格式
wchar_t* ETConvertWCHAR(const char * lpszData, int nLen)
{
	// utf8 -> wcs
	wchar_t *lpwsData;
	lpwsData = new wchar_t[nLen + 1];
	if(lpwsData == NULL)
		return NULL;
	memset(lpwsData, 0, (nLen + 1)*sizeof(wchar_t));
	MultiByteToWideChar(CP_UTF8, NULL, lpszData, (int)strlen(lpszData), lpwsData, nLen);
	return lpwsData;
}
#else
char* ConvertToUTF8(const wchar_t * strData)
{
	// wcs -> utf8
	int nLength;
	char* lpszData;

	if(strData == NULL)
		return NULL;

	nLength = (int)wcslen(strData);
	if(nLength == 0)
		return NULL;

	lpszData = new char[nLength + 1];
	if(lpszData == NULL)
		return NULL;
	memset(lpszData,0,nLength + 1);
	wcstombs(lpszData, strData, nLength);

	return lpszData;
}

char* ConvertToUTF8(const wchar_t * strData, int &nLen)
{
	// wcs -> utf8
	char* lpszData;

	if(strData == NULL)
		return NULL;

	nLen = (int)wcslen(strData);
	if(nLen == 0)
		return NULL;

	lpszData = new char[nLen + 1];
	if(lpszData == NULL)
		return NULL;
	memset(lpszData, 0, nLen + 1);
	wcstombs(lpszData, strData, nLen);

	return lpszData;
}

wchar_t* ConvertWCHAR(const char * lpszData, int nLength)
{
	// utf8 -> wcs
	wchar_t *lpwsData;
	lpwsData = new wchar_t[nLength + 1];
	if(lpwsData == NULL)
		return NULL;
	memset(lpwsData, 0, (nLength + 1)*sizeof(wchar_t));
	mbstowcs(lpwsData, lpszData, nLength*sizeof(wchar_t));
	return lpwsData;
}

std::string ConvertString(const wchar_t * strData)
{
	// utf8 -> string
	int nLength;
	char* lpszData;
	std::string sData;

	if(strData == NULL)
		return sData;

	nLength = (int)wcslen(strData);
	if(nLength == 0)
		return sData;

	lpszData = new char[nLength + 1];
	if(lpszData == NULL)
		return sData;
	memset(lpszData,0,nLength + 1);
	wcstombs(lpszData, strData, nLength);

	sData = lpszData;
	delete [] lpszData;
	return sData;
}

string UTF8ConvertASC(const char * lpszData)
{
	string str;
	str = lpszData;

	return str;
}

//将UTF8转换成Unicode格式
wchar_t* ETConvertWCHAR(const char * lpszData, int nLen)
{
	// utf8 -> wcs
	wchar_t *lpwsData;
	lpwsData = new wchar_t[nLen + 1];
	if(lpwsData == NULL)
		return NULL;
	memset(lpwsData, 0, (nLen + 1)*sizeof(wchar_t));
	mbstowcs(lpwsData, lpszData, nLen*sizeof(wchar_t));
	return lpwsData;
}
#endif


//判断是否为数字，0表示含有非数字，1表示十进制数字，2表示十六进制数字，3表示浮点数
int IsNumber(const char* pData)
{
	int i, nType;
	nType = MF_SYS_NUMBER_TYPE_INT;
	for(i = 0;pData[i] != 0;i++)
	{
		if(pData[i] == ('.'))
		{
			if(nType == MF_SYS_NUMBER_TYPE_INT)
			{
				nType = MF_SYS_NUMBER_TYPE_DOUBLE;
			}
			else
			{
				return MF_SYS_NUMBER_TYPE_INVALID;
			}
		}
		else if((pData[i] >= ('0') && pData[i] <= ('9')))
		{
			continue;
		}
		else if((pData[i] == ('X') || pData[i] == ('x')) && i == 1)
		{
			if(pData[0] == ('0'))
			{
				nType = MF_SYS_NUMBER_TYPE_SPECIAL_INT;
			}
			else
			{
				return MF_SYS_NUMBER_TYPE_INVALID;
			}
		}
		else if((pData[i] >= ('A') && pData[i] <= ('F')) || (pData[i] >= ('a') && pData[i] <= ('f')))
		{
			if(nType == MF_SYS_NUMBER_TYPE_INT)
			{
				nType = MF_SYS_NUMBER_TYPE_SPECIAL_INT;
			}
			else if(nType == MF_SYS_NUMBER_TYPE_SPECIAL_INT)
			{
			}
			else
			{
				return MF_SYS_NUMBER_TYPE_INVALID;
			}
		}
		else if(pData[i] == ('K') || pData[i] == ('k') || pData[i] == ('M') || pData[i] == ('m') || 
			pData[i] == ('G') || pData[i] == ('g') || pData[i] == ('T') || pData[i] == ('t'))
		{
			if(pData[i + 1] == 0)
			{
				//只是带单位，返回为特殊整数即可
				return MF_SYS_NUMBER_TYPE_SPECIAL_INT;
			}
			else if(pData[i + 1] == ('B') || pData[i + 1] == ('b'))
			{
				if(pData[i + 2] == 0)
				{
					return MF_SYS_NUMBER_TYPE_SPECIAL_INT;
				}
				else
				{
					return MF_SYS_NUMBER_TYPE_INVALID;
				}
			}
			else
			{
				return MF_SYS_NUMBER_TYPE_INVALID;
			}
		}
		else
		{
			return MF_SYS_NUMBER_TYPE_INVALID;
		}
	}
	return nType;
}

//判断名字规范，1表示前后都带双引号（意味着大小写敏感），2表示正常字段名，我们自动强制转换成大写，0表示不符合命名规范
int CopyNameFromSql(const char* pData, int nDataLen, char * pName)
{
	if(pData[0] == ('"'))
	{
		int nPos;
		nPos = nDataLen - 1;
		if(nPos < 2 || nPos > 32)
		{
			return 0;
		}
		else if(pData[nPos] != ('"'))
		{
			return 0;
		}
		else
		{
			nPos--;
			memcpy(pName, pData + 1, nPos);
			pName[nPos] = 0;
			return 1;
		}
	}
	else if((pData[0] >= ('A') && pData[0] <= ('Z')) || (pData[0] >= ('a') && pData[0] <= ('z')))
	{
		int i;
		for(i = 0;pData[i] != 0 && i < 31;i++)
		{
			if(pData[i] >= ('a') && pData[i] <= ('z'))
			{
				pName[i] = pData[i] - 32;
			}
			else if(pData[i] >= ('A') && pData[i] <= ('Z'))
			{
				pName[i] = pData[i];
			}
			else if(pData[i] >= ('0') && pData[i] <= ('9'))
			{
				pName[i] = pData[i];
			}
			else if(pData[i] == ('_') || pData[i] == ('$') || pData[i] == ('#'))
			{
				pName[i] = pData[i];
			}
			else
			{
				return 0;
			}
		}
		pName[i] = 0;
		return 2;
	}
	else
	{
		return 0;
	}
}

int CopyUserNameFromSql(const char* pData, int nDataLen, char * pName)
{
	if(pData[0] == ('"'))
	{
		int nPos;
		nPos = nDataLen - 1;
		if(nPos < 2 || nPos > 32)
		{
			return 0;
		}
		else if(pData[nPos] != ('"'))
		{
			return 0;
		}
		else
		{
			nPos--;
			memcpy(pName, pData + 1, nPos);
			pName[nPos] = 0;
			return 1;
		}
	}
	else if((pData[0] >= ('A') && pData[0] <= ('Z')) || (pData[0] >= ('a') && pData[0] <= ('z')))
	{
		int i;
		for(i = 0;pData[i] != 0 && i < 31;i++)
		{
			if(pData[i] >= ('a') && pData[i] <= ('z'))
			{
				pName[i] = pData[i];
			}
			else if(pData[i] >= ('A') && pData[i] <= ('Z'))
			{
				pName[i] = pData[i];
			}
			else if(pData[i] >= ('0') && pData[i] <= ('9'))
			{
				pName[i] = pData[i];
			}
			else if(pData[i] == ('_') || pData[i] == ('$') || pData[i] == ('#'))
			{
				pName[i] = pData[i];
			}
			else
			{
				return 0;
			}
		}
		pName[i] = 0;
		return 2;
	}
	else
	{
		return 0;
	}
}

//重载CopyNameFromSql只做判断用
int CopyNameFromSql(char* pData, int nDataLen)
{
	char * pName = new char[nDataLen];
	if(pData[0] == ('"'))
	{
		int nPos;
		nPos = nDataLen - 1;
		if(nPos < 2 || nPos > 32)
		{
			delete [] pName;
			return 0;
		}
		else if(pData[nPos] != ('"'))
		{
			delete [] pName;
			return 0;
		}
		else
		{
			nPos--;
			memcpy(pName, pData + 1, nPos);
			pName[nPos] = 0;
			delete [] pName;
			return 1;
		}
	}
	else if((pData[0] >= ('A') && pData[0] <= ('Z')) || (pData[0] >= ('a') && pData[0] <= ('z')))
	{
		int i;
		for(i = 0;pData[i] != 0 && i < 31;i++)
		{
			if(pData[i] >= ('a') && pData[i] <= ('z'))
			{
				pName[i] = pData[i] - 32;
			}
			else if(pData[i] >= ('A') && pData[i] <= ('Z'))
			{
				pName[i] = pData[i];
			}
			else if(pData[i] >= ('0') && pData[i] <= ('9'))
			{
				pName[i] = pData[i];
			}
			else if(pData[i] == ('_') || pData[i] == ('$') || pData[i] == ('#'))
			{
				pName[i] = pData[i];
			}
			else
			{
				delete [] pName;
				return 0;
			}
		}
		pName[i] = 0;
		delete [] pName;
		return 2;
	}
	else
	{
		delete [] pName;
		return 0;
	}
}

//判断字符串规范，1表示按正确规范写的字符串（前后都带单引号），2表示传入的字符串太长，0表示不符合规范
int CopyStringFromSql(const char* pData, int nDataLen, char * pName, int nSize)
{
	if(nDataLen <= 1)
	{
		//为空怎么可能呢？
		return 0;
	}
	else if(pData[0] == '\'')
	{
		if(nDataLen - 1 >= nSize)
		{
			//长度不够错误
			return 2;
		}
		else
		{
			if(pData[nDataLen - 1] == '\'')
			{
				memcpy(pName, pData + 1, nDataLen - 2);
				pName[nDataLen - 2] = 0;
				return 1;
			}
		}
	}
	return 0;
}

//重载CopyStringFromSql
int CopyStringFromSql(const char* lpszData, int nDataLen, VARDATA& varValue)
{
	if(nDataLen <= 1)
	{
		//为空怎么可能呢？
		return 0;
	}

	//传入参数lpszData具有两个单引号，此处要去掉这两个单引号
	//nDataLen - 1是因为nDataLen包含两个单引号，而此处要去掉单引号并且存入一个结束符0
	char* lpszBuffer = NULL;
	lpszBuffer = new char[nDataLen - 1];
	memset(lpszBuffer, 0, nDataLen - 1);
	
	if(lpszData[0] == _T('\''))
	{
		if(lpszData[nDataLen - 1] == _T('\''))
		{
			memcpy(lpszBuffer, lpszData + 1, nDataLen - 2);
			lpszBuffer[nDataLen - 2] = 0;
	
			varValue.AttachBuffer(MF_VARDATA_STRING, lpszBuffer, nDataLen - 1);
			return 1;
		}	
	}
	return 0;
}

//重载CopyStringFromSql
int CheckValidStr(const char* lpszData, int nDataLen)
{
	if(nDataLen <= 1)
	{
		//为空怎么可能呢？
		return 0;
	}
	else if(lpszData[0] == _T('\''))
	{
		if(lpszData[nDataLen - 1] == _T('\''))
		{
			return 1;
		}	
	}
	return 0;
}

int CopyPassWordFromSql(const char* pData, int nDataLen, BYTE* pPassWord)
{
	if((pData[0] >= ('A') && pData[0] <= ('Z')) || (pData[0] >= ('a') && pData[0] <= ('z')))
	{
		int i;
		for(i = 0;pData[i] != 0 && i < 31;i++)
		{
			if(pData[i] >= ('a') && pData[i] <= ('z'))
			{
				pPassWord[i] = pData[i];
			}
			else if(pData[i] >= ('A') && pData[i] <= ('Z'))
			{
				pPassWord[i] = pData[i];
			}
			else if(pData[i] >= ('0') && pData[i] <= ('9'))
			{
				pPassWord[i] = pData[i];
			}
			else if(pData[i] == ('_') || pData[i] == ('$') || pData[i] == ('#'))
			{
				pPassWord[i] = pData[i];
			}
			else
			{
				return 0;
			}
		}
		return 1;
	}
	else
	{
		return 0;
	}
}

long long ToNumber(const char* pData, int nDataSize)
{
	char cData;
	char pBuf[64];
	long long nMagnitude, nValue;
	try
	{
		if(nDataSize == 0)
		{
			return 0;
		}
		else if(nDataSize > 62)
		{
			return 0;
		}
		nValue		= 0;
		nMagnitude	= 1;
		cData = pData[nDataSize - 1];
		if(cData >= 'a' && cData <= 'z')
		{
			cData = cData - 'a' + 'A';
		}
		if(cData == _T('B') && nDataSize > 2)
		{
			cData = pData[nDataSize - 2];
			if(cData >= 'a' && cData <= 'z')
			{
				cData = cData - 'a' + 'A';
			}

			if(cData == _T('M'))
			{
				nMagnitude = 1024*1024;
			}
			else if(cData == _T('G'))
			{
				nMagnitude = 1024*1024*1024;
			}
			else if(cData == _T('K'))
			{
				nMagnitude = 1024;
			}
			else if(cData == _T('T'))
			{
				nMagnitude = (long long)1024*1024*1024*1024;
			}
			else
			{
				//可能为十六进制
				if(IsNumber(pData) == 0)
				{
					return 0;
				}
				else
				{
					return _atoi64(pData);
				}
			}

			memset(pBuf, 0, sizeof(pBuf));
			memcpy(pBuf, pData, nDataSize - 2);
			if(IsNumber(pBuf) == 0)
			{
				return 0;
			}
			nValue = _atoi64(pBuf);
			nValue = nValue * nMagnitude;
			return nValue;
		}
		else if(cData == _T('M'))
		{
			nMagnitude = 1024*1024;
		}
		else if(cData == _T('G'))
		{
			nMagnitude = 1024*1024*1024;
		}
		else if(cData == _T('K'))
		{
			nMagnitude = 1024;
		}
		else if(cData == _T('T'))
		{
			nMagnitude = (long long)1024*1024*1024*1024;
		}
		else
		{
			nMagnitude = 1;
		}

		memset(pBuf, 0, sizeof(pBuf));
		if(cData >= _T('0') && cData <= _T('9'))
		{
			memcpy(pBuf, pData, nDataSize);
		}
		else if(cData >= _T('A') && cData <= _T('F'))
		{
			if(IsNumber(pData) == 2)
			{
				return _atoi64(pData);
			}
		}
		else
		{
			memcpy(pBuf, pData, nDataSize - 1);
		}

		if(IsNumber(pBuf) == 0)
		{
			return 0;
		}
		nValue = _atoi64(pBuf);
		nValue = nValue * nMagnitude;
		return nValue;
	}
	catch(...)
	{
	}
	return 0;
}

double ToDouble(const char* pData)
{
	try
	{
		if(IsNumber(pData) != 0)
		{
			return atof(pData);
		}
	}
	catch(...)
	{
	}
	return 0;
}

int GetIndexType(MF_SYS_FILETYPE bFileType, MF_SYS_FIELDTYPE bFieldType, MF_SYS_INDEXTYPE &bIndexType)
{
	bIndexType = 0;
	if(bFileType == MF_SYS_FILETYPE_TREEINDEXFILE)
	{
		switch(bFieldType)
		{
		case MF_SYS_FIELDTYPE_INT:
			bIndexType = MF_SYS_INDEXTYPE_TREE_INT;
			break;
		case MF_SYS_FIELDTYPE_BIGINT:
			bIndexType = MF_SYS_INDEXTYPE_TREE_BIGINT;
			break;
		case MF_SYS_FIELDTYPE_DOUBLE:
		case MF_SYS_FIELDTYPE_DATE:
			bIndexType = MF_SYS_INDEXTYPE_TREE_DOUBLE;
			break;
		case MF_SYS_FIELDTYPE_CHAR:
		case MF_SYS_FIELDTYPE_VARCHAR:
			bIndexType = MF_SYS_INDEXTYPE_FUZZY_CHAR;
			break;
		default:
			return MF_SYS_OBJECT_INDEX_INVALID_FIELDTYPE_ERROR;
		}
	}
	else if(bFileType == MF_SYS_FILETYPE_KVINDEXFILE)
	{
		switch(bFieldType)
		{
		case MF_SYS_FIELDTYPE_INT:
			bIndexType = MF_SYS_INDEXTYPE_KV_INT;
			break;
		case MF_SYS_FIELDTYPE_BIGINT:
			bIndexType = MF_SYS_INDEXTYPE_KV_BIGINT;
			break;
		case MF_SYS_FIELDTYPE_DOUBLE:
		case MF_SYS_FIELDTYPE_DATE:
			bIndexType = MF_SYS_INDEXTYPE_KV_DOUBLE;
			break;
		case MF_SYS_FIELDTYPE_CHAR:
			//只有CHAR可以创建KV索引
			bIndexType = MF_SYS_INDEXTYPE_KV_CHAR;
			break;
		default:
			return MF_SYS_OBJECT_INDEX_INVALID_FIELDTYPE_ERROR;
		}
	}
	else if(bFileType == MF_SYS_FILETYPE_HBTREEINDEXFILE)
	{
		switch(bFieldType)
		{
		case MF_SYS_FIELDTYPE_INT:
			bIndexType = MF_SYS_INDEXTYPE_HB_INT;
			break;
		case MF_SYS_FIELDTYPE_CHAR:
			//只有CHAR可以创建HB树索引
			bIndexType = MF_SYS_INDEXTYPE_HB_CHAR;
			break;
		default:
			return MF_SYS_OBJECT_INDEX_INVALID_FILETYPE_ERROR;
		}
	}
	else if(bFileType == MF_SYS_FILETYPE_SYSFILE || bFileType == MF_SYS_FILETYPE_DATAFILE)
	{
		return MF_SYS_OBJECT_INDEX_INVALID_FILETYPE_ERROR;
	}
	else
	{
		return MF_SYS_OBJECT_INDEX_NOSUPPORT_FILETYPE_ERROR;
	}
	return MF_OK;
}

//获取数据Buffer的长度
int GetFieldBufferLen(int& nLen, MF_SYS_FIELDTYPE bType, int nStringLen)
{
	nLen = 0;
	switch(bType)
	{
	case MF_SYS_FIELDTYPE_NULL:
		nLen = 0;
		break;
	case MF_SYS_FIELDTYPE_INT:
		nLen = sizeof(BYTE) + sizeof(int);
		break;
	case MF_SYS_FIELDTYPE_BIGINT:
		nLen = sizeof(BYTE) + sizeof(long long);
		break;
	case MF_SYS_FIELDTYPE_DOUBLE:
	case MF_SYS_FIELDTYPE_DATE:
		nLen = sizeof(BYTE) + sizeof(double);
		break;
	case MF_SYS_FIELDTYPE_CHAR:
	case MF_SYS_FIELDTYPE_VARCHAR:
	case MF_SYS_FIELDTYPE_CLOB:
	case MF_SYS_FIELDTYPE_BLOB:
		if(nStringLen < 250)
		{
			nLen = sizeof(BYTE) + sizeof(BYTE) + nStringLen;
		}
		else
		{
			nLen = sizeof(BYTE) + sizeof(BYTE) + sizeof(int) + nStringLen;
		}
		break;
	default:
		return MF_COMMON_INVALID_DATATYPE;
	}
	return MF_OK;
}

int GetValueFromFieldBuffer(BYTE bFieldType, LPBYTE lpFieldBuffer, VARDATA& varValue)
{
	BYTE bLen;
	int nDataLen;
	lpFieldBuffer += sizeof(BYTE);
	switch(bFieldType)
	{
	case MF_SYS_FIELDTYPE_INT:
		varValue.SetData(*(int*)lpFieldBuffer);
		break;
	case MF_SYS_FIELDTYPE_BIGINT:
		varValue.SetData(*(long long*)lpFieldBuffer);
		break;
	case MF_SYS_FIELDTYPE_DOUBLE:
		varValue.SetData(*(double*)lpFieldBuffer, MF_SYS_FIELDTYPE_DOUBLE);
		break;
	case MF_SYS_FIELDTYPE_DATE:
		varValue.SetData(*(DATE*)lpFieldBuffer, MF_SYS_FIELDTYPE_DATE);
		break;
	case MF_SYS_FIELDTYPE_CHAR:
	case MF_SYS_FIELDTYPE_VARCHAR:
	case MF_SYS_FIELDTYPE_CLOB:
		//获取字段长度
		bLen = *(BYTE*)lpFieldBuffer;
		if(bLen != 255)
		{
			nDataLen		= bLen;
			lpFieldBuffer	+= sizeof(BYTE);
		}
		else
		{
			lpFieldBuffer	+= sizeof(BYTE);
			nDataLen		= *(int*)lpFieldBuffer;
			lpFieldBuffer	+= sizeof(int);
		}
		varValue.SetData((char*)lpFieldBuffer, nDataLen);
		break;
	case MF_SYS_FIELDTYPE_BLOB:
		bLen = *(BYTE*)lpFieldBuffer;
		if(bLen != 255)
		{
			nDataLen		= bLen;
			lpFieldBuffer	+= sizeof(BYTE);
		}
		else
		{
			lpFieldBuffer	+= sizeof(BYTE);
			nDataLen		= *(int*)lpFieldBuffer;
			lpFieldBuffer	+= sizeof(int);
		}
		varValue.SetBinaryData((LPBYTE)lpFieldBuffer, nDataLen);
		break;
	}
	return MF_OK;
}

//循环左移:
#define CROL(value, bits) ((value << bits) | (value >> (0x20 - bits)))
//循环右移:
#define CROR(value, bits) ((value >> bits) | (value << (0x20 - bits)))


// 获取文件路径的hash值
DWORD CalcSQLHash(const char * lpszSql, int nLen)
{
	LPDWORD lpData;
	DWORD dwHash;
	int i, nCount, nBit;
	dwHash = 0;
	nCount = nLen/4;
	lpData = (LPDWORD)lpszSql;
	for(i = 0;i < nCount;i++)
	{
		nBit = i%30;
		dwHash += (lpData[i] << nBit) | (lpData[i] >> (0x20 - nBit));
	}

	//把最后几个字节处理上去
	if(nLen%4 != 0)
	{
		int j;
		DWORD dwTemp;
		char * lpTemp;
		dwTemp = 0;
		lpTemp = (char *)&dwTemp;
		nBit   = nCount%30;
		for(i = nCount*4, j = 0; i < nLen && i < 4; i++, j++)
		{
			lpTemp[j] = lpszSql[i];
		}
		dwHash += (dwTemp << nBit) | (dwTemp >> (0x20 - nBit));
	}
	return dwHash;
}

DWORD BKDRHash(char *str, int nLen)
{
	unsigned int seed = 131; // 31 131 1313 13131 131313 etc..
	unsigned int hash = 0;
	
	while (nLen)
	{
		hash = hash * seed + (*str++);
		nLen--;
	}

	return (hash & 0x7FFFFFFF);
}
// 获取文件路径的hash值
DWORD CalcKeyHash(LPBYTE lpKey, int nHashSize)
{
	char * lpData;
	UINT  nHash, i;
	ULONGLONG ullDigitalHash;
	if(nHashSize == 0)
	{
		return 0;
	}

	lpData			 = (char *)lpKey;
	ullDigitalHash	 = 0xd3202b73;
	for(i = 0; lpKey[i] != 0; i++)
	{
		if(lpData[i] >= '0' && lpData[i] <= '9')
		{
			ullDigitalHash = ullDigitalHash * 10 + lpData[i] - '0';
		}
		else if(lpData[i] >= 'A' && lpData[i] <= 'Z')
		{
			ullDigitalHash = ullDigitalHash * 37 + lpData[i] - 'A' + 10;
		}
		else if(lpData[i] >= 'a' && lpData[i] <= 'z')
		{
			ullDigitalHash = ullDigitalHash * 37 + lpData[i] - 'a' + 10;
		}
		else
		{
			ullDigitalHash = ullDigitalHash * 131 + lpKey[i];
		}
	}

	nHash			= (DWORD)(ullDigitalHash % nHashSize);
	return nHash;
}
DWORD CalcKeyHash(long long nKey, int nHashSize)
{
	int nHashNodeNo;
	nHashNodeNo = nKey % nHashSize;
	return nHashNodeNo;
}

//注意：nLen是lpBuf的长度，注意与CharConvertHex函数的nBufsize区分
int HexConvertChar(const LPBYTE lpBuf, const int nLen, char *lpszData)
{
	int i,j;
	BYTE bData, bTemp;
	for(i = 0,j = 0;i < nLen;i++)
	{
		bData = lpBuf[i];
		bTemp = bData&0XF0;					//取高4位
		bTemp = bTemp >> 4;
		if(bTemp < 10)
		{
			lpszData[j] = '0' + bTemp;
		}
		else
		{
			lpszData[j] = 'A' + bTemp - 10;
		}
		j++;
		bTemp = bData&0XF;					//取低4位
		if(bTemp < 10)
		{
			lpszData[j] = '0' + bTemp;
		}
		else
		{
			lpszData[j] = 'A' + bTemp - 10;
		}
		j++;
	}
	lpszData[j] = 0;
	return MF_OK;
}

//说明：根据16进制转2进制的原则：用4位的二进制来表示1位的16进制
//而现在16进制是以字符串形式存放在XML中的，所以1个16进制的字符长度为1个字节即8位，而8位可以存放2个2进制数据
//所以nBufsize的大小为lpszData的长度除以2
//注意：nBufsize是lpBuf的长度
int CharConvertHex(const char *lpszData, LPBYTE lpBuf, const int nBufsize)
{
	int i, j, nIndex;
	BYTE bData, bTemp;

	i		= 0;
	nIndex	= 0;
	for(j = 0;nIndex < nBufsize && lpszData[j] != 0;j++)
	{
		if(lpszData[j] >= '0' && lpszData[j] <= '9')
		{
			bTemp = lpszData[j] - '0';
		}
		else if(lpszData[j] >= 'A' && lpszData[j] <= 'F')
		{
			bTemp = lpszData[j] - 'A' + 10;
		}
		else if(lpszData[j] >= 'a' && lpszData[j] <= 'f')
		{
			bTemp = lpszData[j] - 'a' + 10;
		}
		else if(lpszData[j] == '\r' || lpszData[j] == '\n' || lpszData[j] == ' ' || lpszData[j] == '	')
		{
			//忽略部分看不见的格式信息
			continue;
		}
		else
		{
			return MF_INNER_SYS_INVALID_PARAMETER_ERROR;
		}

		if(i%2 == 0)
		{
			bData = bTemp << 4;
		}
		else
		{
			bData = bData | bTemp;
			lpBuf[nIndex] = bData;
			nIndex++;
		}
		i++;
	}
	return MF_OK;
}

int HexConvertWChar(const LPBYTE lpBuf, const int nLen, wchar_t *lpszData)
{
	int i,j;
	BYTE bData, bTemp;
	for(i = 0,j = 0;i < nLen;i++)
	{
		bData = lpBuf[i];
		bTemp = bData&0XF0;
		bTemp = bTemp >> 4;
		if(bTemp < 10)
		{
			lpszData[j] = '0' + bTemp;
		}
		else
		{
			lpszData[j] = 'A' + bTemp - 10;
		}
		j++;
		bTemp = bData&0XF;
		if(bTemp < 10)
		{
			lpszData[j] = '0' + bTemp;
		}
		else
		{
			lpszData[j] = 'A' + bTemp - 10;
		}
		j++;
	}
	lpszData[j] = 0;
	return MF_OK;
}

int WCharConvertHex(const wchar_t *lpszData, LPBYTE lpBuf, const int nBufsize)
{
	int i, j, nIndex;
	BYTE bData, bTemp;

	i		= 0;
	nIndex	= 0;
	for(j = 0;nIndex < nBufsize && lpszData[j] != 0;j++)
	{
		if(lpszData[j] >= '0' && lpszData[j] <= '9')
		{
			bTemp = lpszData[j] - '0';
		}
		else if(lpszData[j] >= 'A' && lpszData[j] <= 'F')
		{
			bTemp = lpszData[j] - 'A' + 10;
		}
		else if(lpszData[j] >= 'a' && lpszData[j] <= 'f')
		{
			bTemp = lpszData[j] - 'a' + 10;
		}
		else if(lpszData[j] == '\r' || lpszData[j] == '\n' || lpszData[j] == ' ' || lpszData[j] == '	')
		{
			//忽略部分看不见的格式信息
			continue;
		}
		else
		{
			return MF_INNER_SYS_INVALID_PARAMETER_ERROR;
		}

		if(i%2 == 0)
		{
			bData = bTemp << 4;
		}
		else
		{
			bData = bData | bTemp;
			lpBuf[nIndex] = bData;
			nIndex++;
		}
		i++;
	}
	return MF_OK;
}


long long ConvertTimetoLongLong(struct timeval tTime)
{
	long long lTime;
	lTime = ((long long)tTime.tv_sec + XSEC)*10000000 + (long long)tTime.tv_usec*10;
	return lTime;
}

void ConvertTimetoFileTime(struct timeval tTime, FILETIME &fTime)
{
	long long * lpTime;
	lpTime = (long long *)&fTime;
	*lpTime = ((long long)tTime.tv_sec + XSEC)*10000000 + (long long)tTime.tv_usec*10;
}

struct timeval ConvertFileTimetoTime(FILETIME fTime)
{
	struct timeval tTime;
	long long llTmp, *lpTime;

	lpTime = (long long*)&fTime;
	llTmp = *lpTime - X100NSEC;
	tTime.tv_sec = (int)(llTmp/10000000);
	tTime.tv_usec = (int)(llTmp/10 - (long long)tTime.tv_sec*1000000);
	return tTime;
}

DATE ConvertTimetoDate(time_t tTime)
{
	DATE dtTime;

	//25569是1970-1-1到1899-12-30之间相差的天数
	dtTime = (DATE)(tTime - timezone) / (DAYSECONDS) + 25569;
	return dtTime;
}

time_t ConvertDateToTime(DATE dtTime)
{
	time_t tTime;

	//25569是1970-1-1到1899-12-30之间相差的天数
	tTime = (time_t)((dtTime - 25569)*DAYSECONDS + timezone + 0.5001);
	return tTime;
}

//比较Double类型
int CompareDouble(double dblValue1, double dblValue2)
{
	double dblValue;
	
	dblValue = dblValue1 - dblValue2;
	if(fabs(dblValue) < PRECISION)
	{
		return 0;
	}
	else if(dblValue < 0)
	{
		return -1;
	}
	else
	{
		return 1;
	}
}

int Encryption(BYTE* pSrc, BYTE* pDest)
{
#ifdef WIN32
	MD5_CTX mdContext;
	MD5Init(&mdContext);  
	MD5Update(&mdContext, pSrc, 24);  
	MD5Final(&mdContext);
	memcpy(pDest, mdContext.digest, 16);
#else
	memcpy(pDest, pSrc, 24);
#endif
	return MF_OK;
}

char * GetErrorDescription(int nErrorNO)
{
	int nCount, i;
	nCount = sizeof(g_pVernoxErrorDef)/sizeof(VERNOXERROR_DEF);
	for(i = 0;i < nCount;i++)
	{
		if(g_pVernoxErrorDef[i].nErrorNO == nErrorNO)
		{
			return g_pVernoxErrorDef[i].lpszDescription;
		}
	}
	return "未知错误码";
}

wchar_t * GetErrorWDescription(int nErrorNO)
{
	int nCount, i;
	nCount = sizeof(g_pwVernoxErrorDef)/sizeof(WVERNOXERROR_DEF);
	for(i = 0;i < nCount;i++)
	{
		if(g_pwVernoxErrorDef[i].nErrorNO == nErrorNO)
		{
			return g_pwVernoxErrorDef[i].lpwszDescription;
		}
	}
	return L"未知错误码";
}

