/*
 *    Copyright (C) 2017 Sobey Inc.
 *
 *    This program is free software: you can redistribute it and/or  modify
 *    it under the terms of the GNU Affero General Public License, version 3,
 *    as published by the Free Software Foundation.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Affero General Public License for more details.
 *
 *    You should have received a copy of the GNU Affero General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "stdafx.h"
#include "MemDBInterface.h"
#include "io.h"

#define CONNECT_THRESHOLD  5
extern HINSTANCE g_hVernoxInstance;
// CMemDBInterface
CMemDBInterface::CMemDBInterface()
{
	m_bStatic		      = FALSE;
	m_nNodeNum			  = 0;
	m_lInterfaceType	  = 0;
	m_lWriteEfficiencyLog = 0;
	m_uServiceTCPIP		  = inet_addr("127.0.0.1");
	//m_uServiceTCPIP	      = inet_addr("172.16.131.20");
	m_uServiceTCPPort	  = htons(8000);
	m_skSocket			  = INVALID_SOCKET;
	m_bRandomFactor		  = 0;
	m_bOpenFlag			  = FALSE;

	memset(m_lpConnectInfo, 0, 10*sizeof(CONNECTINFO));
	::InitializeCriticalSection(&m_csInterFace); 
}

CMemDBInterface::~CMemDBInterface()
{
	::DeleteCriticalSection(&m_csInterFace);
}

HRESULT CMemDBInterface::ExecuteCommandCore(BSTR bstrCmd, LONG* plAffectCount)
{
	HRESULT hr;
	__try
	{
		if(!m_bOpenFlag)
		{
			return E_FAIL;
		}
		//ʹTCPʽӷ
		hr = TCPExecuteCommand(bstrCmd, plAffectCount);
	}
	__except(MyExceptionHandler(GetExceptionCode(), GetExceptionInformation()))
	{
		Trace(_T("ExecuteCommand"), 0, 999001001, _T("ݿִ쳣ֵ0X%XbstrCmd%sӿͣ%d"), hr, bstrCmd, m_lInterfaceType);
		return E_FAIL;
	}
	return hr;
}

STDMETHODIMP CMemDBInterface::ExecuteCommand(BSTR bstrCmd, LONG* plAffectCount)
{
	HRESULT hr;
	DWORD dwTime;
	LARGE_INTEGER liFrequence, liStart, liEnd;
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	QueryPerformanceFrequency(&liFrequence);
	QueryPerformanceCounter(&liStart);
	hr = ExecuteCommandCore(bstrCmd, plAffectCount);
	QueryPerformanceCounter(&liEnd);
	if(m_lWriteEfficiencyLog == 1)
	{
		dwTime = (DWORD)(1000000 * (liEnd.QuadPart - liStart.QuadPart) / liFrequence.QuadPart); //㵽΢
		Trace(_T("ExecuteCommand"), 0, 999001009, _T("ݿִнʱ%d΢룬ֵ0X%XӰ%dbstrCmd%sӿͣ%d"), dwTime, hr, *plAffectCount, bstrCmd, m_lInterfaceType);
	}

	return hr;
}

HRESULT CMemDBInterface::GetRecordsetCore(BSTR bstrSql, _Recordset ** ppRs)
{
	HRESULT hr;
	__try
	{
		if(!m_bOpenFlag)
		{
			return E_FAIL;
		}
		//ʹTCPʽӷ
		hr = TCPGetRecordset(bstrSql, ppRs);
	}
	__except(MyExceptionHandler(GetExceptionCode(), GetExceptionInformation()))
	{
		Trace(_T("GetRecordset"), 0, 999002001, _T("ݿִ쳣ֵ0X%XbstrSql%sӿͣ%d"), hr, bstrSql, m_lInterfaceType);
		return E_FAIL;
	}
	return hr;
}

STDMETHODIMP CMemDBInterface::GetRecordset(BSTR bstrSql, _Recordset ** ppRs)
{
	HRESULT hr;
	DWORD dwTime, lRecordCount;
	LARGE_INTEGER liFrequence, liStart, liEnd;
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	QueryPerformanceFrequency(&liFrequence);
	QueryPerformanceCounter(&liStart);
	hr = GetRecordsetCore(bstrSql, ppRs);
	QueryPerformanceCounter(&liEnd);
	if(m_lWriteEfficiencyLog == 1)
	{
		dwTime = (DWORD)(1000000 * (liEnd.QuadPart - liStart.QuadPart) / liFrequence.QuadPart); //㵽΢
		if (SUCCEEDED(hr) && ppRs && *ppRs)
			lRecordCount = (*ppRs)->RecordCount;
		else
			lRecordCount = 0;
		Trace(_T("GetRecordset"), 0, 999002009, _T("ݿִнʱ%d΢룬ֵ0X%X%dbstrSql%sӿͣ%d"), dwTime, hr, lRecordCount, bstrSql, m_lInterfaceType);
	}

	return hr;
}

HRESULT CMemDBInterface::UpdateRecordsetCore(_Recordset * pRs)
{
	HRESULT hr;
	__try
	{
		if(!m_bOpenFlag)
		{
			return E_FAIL;
		}
		//ʹTCPʽӷ
		hr = TCPUpdateRecordset(pRs);
	}
	__except(MyExceptionHandler(GetExceptionCode(), GetExceptionInformation()))
	{
		Trace(_T("UpdateRecordset"), 0, 999003001, _T("ݿ޸ִ쳣ֵ0X%Xӿͣ%d"), hr, m_lInterfaceType);
		return E_FAIL;
	}
	return hr;

}

BYTE CMemDBInterface::CreateRandomFactor()
{
	GUID guid;
	LPBYTE lpBuf;
	lpBuf = (LPBYTE)&guid;
	while(TRUE)
	{
		::CoCreateGuid(&guid);
		//ȥͬ
		if(lpBuf[0] != m_bRandomFactor && lpBuf[0] != 0)
			break;
	}
	m_bRandomFactor = lpBuf[0];
	return m_bRandomFactor;
}

STDMETHODIMP CMemDBInterface::UpdateRecordset(_Recordset * pRs)
{
	HRESULT hr;
	DWORD dwTime;
	LARGE_INTEGER liFrequence, liStart, liEnd;
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	if(pRs == NULL)
	{
		Trace0(_T("UpdateRecordset"), 0, 999003002, _T("pRsΪ"));
		return E_FAIL;
	}

	QueryPerformanceFrequency(&liFrequence);
	QueryPerformanceCounter(&liStart);
	hr = UpdateRecordsetCore(pRs);
	QueryPerformanceCounter(&liEnd);

	if(m_lWriteEfficiencyLog == 1)
	{
		long lRecordCount = 0;
		_bstr_t bstrTableName = L"";
		try
		{
			bstrTableName = pRs->Fields->GetItem(0L)->Properties->GetItem(L"BASETABLENAME")->Value;
			lRecordCount = pRs->RecordCount;
		}
		catch (_com_error)
		{
		}
		dwTime = (DWORD)(1000000 * (liEnd.QuadPart - liStart.QuadPart) / liFrequence.QuadPart); //㵽΢
		Trace(_T("UpdateRecordset"), 0, 999003009, _T("ݿ޸ִнʱ%d΢룬ֵ0X%X޸ı%s޸%dӿͣ%d"), dwTime, hr, (LPCTSTR)bstrTableName, lRecordCount, m_lInterfaceType);
	}
	return hr;
}

void CMemDBInterface::GetSocketInfo(ULONG &ulIP, USHORT &usPort)
{
	int i, nLen;
	ULONG uLocalIP;
	sockaddr_in sIn;
	nLen	 = sizeof(sockaddr_in);
	uLocalIP = ntohl(inet_addr("127.0.0.1"));
	memset(&sIn, 0, nLen);
	getsockname(m_skSocket, (sockaddr*)&sIn, &nLen);
	usPort	= ntohs(sIn.sin_port);
	ulIP	= ntohl(sIn.sin_addr.s_addr);
	if(ulIP == uLocalIP)
	{
		hostent *phostent;
		char szHostName[256];
		memset(szHostName, 0, 256);
		if(gethostname(szHostName, 256) == SOCKET_ERROR)
		{
			return ;
		}

		phostent = gethostbyname(szHostName);
		if(phostent == NULL)
		{
			return ;
		}
		for(i = 0; phostent != NULL && phostent->h_addr_list[i] != NULL; i++)
		{
			memcpy(&ulIP, phostent->h_addr_list[i], 4);
			ulIP = ntohl(ulIP);
			if(ulIP != uLocalIP)
			{
				break;
			}
		}
	}
}

int CMemDBInterface::UserLogin()
{
	BYTE bToken;
	char lpFilePath[MAX_PATH];
	MF_CLIENTINFO *lpClientInfo;
	MF_SERVICE_TCP_COMMAND stTcpCmd;
	int i, nRet, nSize, nSocketError, nIndex;

	bToken = CreateRandomFactor();
	nSize = sizeof(MF_SERVICE_TCP_COMMAND);
	memset(&stTcpCmd, 0, nSize);

	//ͿͻϢû¼Ϣ
	stTcpCmd.m_bVersion		= 1;
	stTcpCmd.m_bToken		= bToken;
	stTcpCmd.m_bType		= MF_INTERFACE_COMMAND_USERLOGIN;
	stTcpCmd.m_lDataLength	= 0;
	stTcpCmd.m_lDigitalData	= 0;
	lpClientInfo = (MF_CLIENTINFO*)stTcpCmd.m_lpszParam;

	lpClientInfo->m_bCmdType     = 1;
	GetModuleFileNameA(NULL, lpFilePath, 256);
	nIndex = (int)strlen(lpFilePath) - 1;
	for(;nIndex > 0;nIndex--)
	{
		if(lpFilePath[nIndex] == _T('\\'))
		{
			memcpy(lpClientInfo->m_lpszProgramName, lpFilePath + nIndex + 1, 23);
			break;
		}
	}
	lpClientInfo->m_dwProcessID = GetCurrentProcessId();
	lpClientInfo->m_dwThreadID  = GetCurrentThreadId();
	lpClientInfo->m_ullUserData = 0;
	lpClientInfo->m_dwUserData  = 0;
	GetSocketInfo(lpClientInfo->m_ulIP, lpClientInfo->m_usPort);

	memcpy(lpClientInfo->m_pUserName, m_pUserName, 23);
	memcpy(lpClientInfo->m_pPassword, m_pPassword, 23);
	nRet = send(m_skSocket, (char *)&stTcpCmd , nSize, 0);
	if(nRet  != nSize)
	{
		nSocketError = WSAGetLastError();
		Trace(_T("UserLogin"), 0, 888002001, _T("ͨTCP˷ʧܣֵ%d룺%d"), nRet, nSocketError);
		return nSocketError;
	}
	//շ˷
	i = 0;
	while(TRUE)
	{
		memset(&stTcpCmd, 0, sizeof(MF_SERVICE_TCP_COMMAND));
		nRet = recv(m_skSocket, (char *)&stTcpCmd, sizeof(MF_SERVICE_TCP_COMMAND), 0);
		if(nRet == 0)
		{
			//ӱر
			nSocketError = WSAGetLastError();
			Trace(_T("UserLogin"), 0, 888002002, _T("շʱӱرյĴ󣬷ֵ%d룺%d"), nRet, nSocketError);
			return nSocketError;
		}
		else if (nRet == SOCKET_ERROR)
		{
			//յSocket
			nSocketError = WSAGetLastError();
			Trace(_T("UserLogin"), 0, 888002003, _T("շʱSocket󣬷ֵ%d룺%d"), nRet, nSocketError);
			return nSocketError;
		}
		else if(nRet == sizeof(MF_SERVICE_TCP_COMMAND))
		{
			//жϷصǷȷ
			if(stTcpCmd.m_bType == MF_INTERFACE_TCPCOMMAND_TEST)
			{
				continue;
			}
			else if(stTcpCmd.m_bToken != bToken)
			{
				i++;
				if(i < 5)
				{
					if(stTcpCmd.m_lDataLength == 0)
					{
						continue;
					}
					else
					{
						char* lpBuffer;
						lpBuffer = new char[stTcpCmd.m_lDataLength];
						if(lpBuffer != NULL)
						{
							ReceiveBuffer(lpBuffer, stTcpCmd.m_lDataLength);
							delete [] lpBuffer;
							continue;
						}
						else
						{
							//Ѿû취Զָ
							ClearSocket();
							break;
						}
					}
				}
				else
				{
					break;
				}
			}
			else
			{
				break;
			}
		}
		else
		{
			nSocketError = WSAGetLastError();
			Trace(_T("UserLogin"), 0, 888002004, _T("շʱֵģпǷ˵ȴʱͻԶTCPݣֵ%d룺%d"), nRet, nSocketError);
			return nSocketError;
		}
	}

	//жϷ
	if(stTcpCmd.m_bType == MF_INTERFACE_TCPCOMMAND_FAIL)
	{
		//Ϊʧܽ
		Trace(_T("UserLogin"), 0, 888002005, _T("û¼ʧܣû%s"), m_pUserName);
		return stTcpCmd.m_lDigitalData;
	}
	else if(stTcpCmd.m_bType == MF_INTERFACE_TCPCOMMAND_OK)
	{
		return S_OK;
	}
	else
	{
		//ķֵ
		ClearSocket();
		Trace(_T("UserLogin"), 0, 888002006, _T("˷صͲȷͻԶTCPݣյͣ%d"), stTcpCmd.m_bType);
		return MF_FAILED;
	}
	return MF_FAILED;
}


//߳IDȡһõ׽
int CMemDBInterface::Connect(BOOL bRetry)
{
	int i;
	CString strIP;
	int nRet, nBufferSize, nNetTimeout, nLastError;
	try
	{
		//ַṹĿǰSDKжΪ
		SOCKADDR_IN sa;
		memset(&sa, 0, sizeof(sa));
		sa.sin_family		=AF_INET;
		sa.sin_port			=m_uServiceTCPPort;
		sa.sin_addr.s_addr	=m_uServiceTCPIP;
		
		//׽
		m_skSocket = socket(AF_INET,SOCK_STREAM, 0);
		if(m_skSocket == INVALID_SOCKET)
		{
			if(WSANOTINITIALISED == WSAGetLastError())
			{
				WORD		wVersionRequested;   
				WSADATA		wsaData;
				wVersionRequested = MAKEWORD(2, 0);
				WSAStartup(wVersionRequested, &wsaData);
				m_skSocket = socket(AF_INET,SOCK_STREAM, 0);
				if(m_skSocket == INVALID_SOCKET)
				{
					Trace(_T("Connect"), 0, 999004001, _T("socketʧܣ룺%d"), WSAGetLastError());
					return MF_INTERFACE_SOCKET_CREATE_FAILED;
				}
			}
		}
		
		// ָ
		nRet = connect(m_skSocket,(const sockaddr*)&sa,sizeof(sa));
		if(nRet == SOCKET_ERROR)
		{
			if(!bRetry)
			{
				nLastError = WSAGetLastError();
				strIP.Format(_T("%d.%d.%d.%d"), FIRST_IPADDRESS(m_uServiceTCPIP),SECOND_IPADDRESS(m_uServiceTCPIP), THIRD_IPADDRESS(m_uServiceTCPIP), FOURTH_IPADDRESS(m_uServiceTCPIP));
				Trace(_T("Connect"), 0, 999004002, _T("ӷʧܣ룺%dIP%s˿ڣ%d"), nLastError, strIP, m_uServiceTCPPort);
				return MF_INTERFACE_SOCKET_CONNECT_FAILED;
			}
			//ӣԭ
			i = 0;
			while(TRUE)
			{
				Sleep(100);
				nRet = connect(m_skSocket,(const sockaddr*)&sa,sizeof(sa));
				if(nRet == SOCKET_ERROR)
				{
					i++;
					if(i > 3)
					{
						nLastError = WSAGetLastError();
						strIP.Format(_T("%d.%d.%d.%d"), FIRST_IPADDRESS(m_uServiceTCPIP),SECOND_IPADDRESS(m_uServiceTCPIP), THIRD_IPADDRESS(m_uServiceTCPIP), FOURTH_IPADDRESS(m_uServiceTCPIP));
						Trace(_T("Connect"), 0, 999004003, _T("ӷʧܣ룺%dIP%s˿ڣ%d"), nLastError, strIP, m_uServiceTCPPort);
						return MF_INTERFACE_SOCKET_CONNECT_FAILED;
					}
				}
				else
				{
					break;
				}
			}
		}

		//վ飬BufferΪ2MBufferΪ32KԵõЧ
		nBufferSize = 1024*1024;
		setsockopt(m_skSocket, SOL_SOCKET, SO_SNDBUF,(const char FAR *)&nBufferSize, sizeof(nBufferSize));
		nBufferSize = 32*1024;
		setsockopt(m_skSocket, SOL_SOCKET, SO_RCVBUF,(const char FAR *)&nBufferSize, sizeof(nBufferSize));

		//TCPNagle㷨TCPӦٶȣĬǹرյ
		const char chOpt=1;
		setsockopt(m_skSocket, IPPROTO_TCP, TCP_NODELAY, &chOpt, sizeof(char));

		//ýճʱ
#ifdef WIN32
		nNetTimeout = INT_MAX;//30룬
		setsockopt(m_skSocket, SOL_SOCKET, SO_SNDTIMEO, (char *)&nNetTimeout, sizeof(int));
		nNetTimeout = INT_MAX;//180룬
		setsockopt(m_skSocket, SOL_SOCKET, SO_RCVTIMEO, (char *)&nNetTimeout, sizeof(int));
#else
		struct timeval timeout;
		timeout.tv_sec = INT_MAX;
		timeout.tv_usec = 0;
		setsockopt(m_skSocket, SOL_SOCKET, SO_SNDTIMEO, (char *)&timeout, sizeof(struct timeval));
		setsockopt(m_skSocket, SOL_SOCKET, SO_RCVTIMEO, (char *)&timeout, sizeof(struct timeval));
#endif

		return MF_OK;
	}
	catch(...)
	{
		Trace(_T("Connect"), 0, 999004099, _T("ͨTCPڴݷʱδ֪쳣룺%d"), GetLastError());
		return MF_INNER_EXPCETION_FAILED;
	}
	return MF_FAILED;
}

int CMemDBInterface::OpenDatabase(BOOL bRetry)
{
	int nRet;
	nRet = ConnectToService(bRetry);
	if (nRet != MF_OK)
	{
		return nRet;
	}
	nRet = UserLogin();
	if (nRet != MF_OK)
	{
		return nRet;
	}
	return MF_OK;
}

int CMemDBInterface::ConnectToService(BOOL bRetry)
{
	return Connect(bRetry);
}

char* CMemDBInterface::ConvertToUTF8(const wchar_t * strData)
{
	// wcs -> utf8
	char * lpszData;
	if(strData == NULL)
		return NULL;
	int nLength = (int)wcslen(strData);
	if(nLength == 0)
		return NULL;
	int nLen = WideCharToMultiByte(CP_UTF8, NULL, strData,  nLength, NULL, NULL, NULL, NULL);
	lpszData = new char[nLen + 1];
	if(lpszData == NULL)
		return NULL;
	memset(lpszData,0,nLen + 1);
	WideCharToMultiByte(CP_UTF8, NULL, strData, nLength, lpszData, nLen, NULL, NULL);
	return lpszData;
}

CString CMemDBInterface::ConvertString(const char * lpszData)
{
	// utf8 -> wcs
	CString strData;
	wchar_t *lpwsData;
	if(lpszData == NULL)
		return NULL;
	int nLength = (int)strlen(lpszData);
	if(nLength == 0)
		return NULL;
	int nLen = MultiByteToWideChar(CP_UTF8, NULL, lpszData, nLength, NULL, NULL);
	lpwsData = new wchar_t[nLen + 1];
	if(lpwsData == NULL)
		return NULL;
	memset(lpwsData, 0, (nLen + 1)*sizeof(wchar_t));
	MultiByteToWideChar(CP_UTF8, NULL, lpszData, nLength, lpwsData, nLen);
	strData = lpwsData;
	delete [] lpwsData;
	return strData;
}

void CMemDBInterface::ConvertBSTR(const char * lpszData, CComBSTR &bstrRs)
{
	// utf8 -> wcs
	wchar_t *lpwsData;
	if(lpszData == NULL)
		return ;
	int nLength = (int)strlen(lpszData);
	if(nLength == 0)
		return ;
	int nLen = MultiByteToWideChar(CP_UTF8, NULL, lpszData, nLength, NULL, NULL);
	lpwsData = new wchar_t[nLen + 1];
	if(lpwsData == NULL)
		return ;
	memset(lpwsData, 0, (nLen + 1)*sizeof(wchar_t));
	MultiByteToWideChar(CP_UTF8, NULL, lpszData, nLength, lpwsData, nLen);
	bstrRs = lpwsData;
	delete [] lpwsData;
}

HRESULT CMemDBInterface::ConvertToRecordset(wchar_t * lpszXml, _Recordset ** ppRs)
{
	HRESULT hr;
	_RecordsetPtr pRs;
	_StreamPtr pStream;
	try
	{
		hr = pStream.CreateInstance(__uuidof(Stream), NULL, CLSCTX_ALL);
		if (FAILED(hr))
		{
			Trace(_T("ConvertToRecordset"), 0, 999005001, _T("Streamʧܣ0X%XδʼOLE!"), hr);
			return E_FAIL;
		}

		hr = pRs.CreateInstance(__uuidof(Recordset), NULL, CLSCTX_ALL);
		if (FAILED(hr))
		{
			Trace(_T("ConvertToRecordset"), 0, 999005002, _T("Recordsetʧܣ0X%X!"), hr);
			return E_FAIL;
		}

		// ַص
		hr = pStream->Open(vtMissing, adModeUnknown, adOpenStreamUnspecified, _T(""), _T(""));
		if (FAILED(hr))
		{
			Trace(_T("ConvertToRecordset"), 0, 999005003, _T("ָʧܣ0X%X!"), hr);
			return E_FAIL;
		}

		hr = pStream->WriteText(lpszXml, adWriteChar);
		if (FAILED(hr))
		{
			Trace(_T("ConvertToRecordset"), 0, 999005004, _T("WriteText ʧܣ0X%XЧXMLݣ"), hr);
			Trace0(_T("ConvertToRecordset"), 0, 999005005, lpszXml);
			return E_FAIL;
		}
		// мxmlRecordsetȥ
		pStream->Position = 0;
		hr = pRs->Open(pStream.GetInterfacePtr(), vtMissing, adOpenStatic, adLockBatchOptimistic, adCmdFile);
		if (FAILED(hr))
		{
			Trace(_T("ConvertToRecordset"), 0, 999005006, _T("pRs->Open յл_Recordsetʧܣ0X%XXMLݣ"), hr);
			Trace0(_T("ConvertToRecordset"), 0, 999005007, lpszXml);
			return E_FAIL;
		}
		*ppRs = pRs.Detach();
		return S_OK;
	}
	catch (_com_error &e)
	{
		Trace(_T("ConvertToRecordset"), 0, 999005008, _T("XMLַתɽʱCOM쳣(0X%X)%s"), e.Error(), (LPCTSTR)e.Description());
		return E_FAIL;
	}
	catch(...)
	{
		Trace(_T("ConvertToRecordset"), 0, 999005009, _T("XMLַתɽʱδ֪쳣%d"), GetLastError());
		return E_FAIL;
	}
	return E_FAIL;
}

HRESULT CMemDBInterface::ConvertFromRecordset(_Recordset * pRs, CComBSTR &btrXml)
{
	HRESULT hr;
	IXMLDOMDocumentPtr pXMLDocInput;
	try
	{
		////Recordset浽XMLԱ㴦
		hr = pXMLDocInput.CreateInstance(__uuidof(DOMDocument));
		if(FAILED(hr))
		{
			Trace(_T("ConvertFromRecordset"), 0, 999006001, _T("DOMDocumentʧܣ룺0X%X"), hr);
			return E_FAIL;
		}

		hr = pRs->Save(pXMLDocInput.GetInterfacePtr(), adPersistXML);
		if (FAILED(hr))
		{
			Trace(_T("ConvertFromRecordset"), 0, 999006002, _T("Recordset浽XMLʧܣ룺0X%X"), hr);
			return E_FAIL;
		}
		hr = pXMLDocInput->get_xml(&btrXml);
		if (FAILED(hr))
		{
			Trace(_T("ConvertFromRecordset"), 0, 999006003, _T("ȡXMLַʧܣ룺0X%X"), hr);
			return E_FAIL;
		}
	}
	catch (_com_error &e)
	{ 
		Trace(_T("ConvertFromRecordset"), 0, 999006004, _T("תXMLַʱCOM쳣(0X%X)%s"), e.Error(), (LPCTSTR)e.Description());
		return E_FAIL;
	}
	catch(...)
	{
		Trace(_T("ConvertToRecordset"), 0, 999006005, _T("תXMLַʱδ֪쳣%d"), GetLastError());
		return E_FAIL;
	}
	return S_OK;
}

void CMemDBInterface::ClearSocket()
{
#ifdef WIN32
	long nRet;
	//ʱԼ500
	int nReceiveTimeOut = 1;
	char lpTempBuffer[1024];
	//սնеݣ޸ĳʱʱ䣬Ȼ޴εĽգֱԺŷ
	setsockopt(m_skSocket, SOL_SOCKET, SO_RCVTIMEO, (char *)&nReceiveTimeOut, sizeof(int));
	while(TRUE)
	{
		nRet = recv(m_skSocket, lpTempBuffer, 1024, 0);
		if(nRet == -1)
		{
			break;
		}
	}
	nReceiveTimeOut = INT_MAX;//180룬
	setsockopt(m_skSocket, SOL_SOCKET, SO_RCVTIMEO, (char *)&nReceiveTimeOut, sizeof(int));
#else
	long nRet;
	//ʱԼ500
	struct timeval timeout;
	char lpTempBuffer[1024];
	//սնеݣ޸ĳʱʱ䣬Ȼ޴εĽգֱԺŷ
	timeout.tv_sec = 0;
	timeout.tv_usec = 1;
	setsockopt(m_skSocket, SOL_SOCKET, SO_RCVTIMEO, (char *)&timeout, sizeof(struct timeval));
	while(TRUE)
	{
		nRet = recv(m_skSocket, lpTempBuffer, 1024, 0);
		if(nRet == -1)
		{
			break;
		}
	}
	timeout.tv_sec = INT_MAX;
	timeout.tv_usec = 0;
	setsockopt(m_skSocket, SOL_SOCKET, SO_RCVTIMEO, (char *)&timeout, sizeof(struct timeval));
#endif
}

//SocketݣҪڴĽ
BOOL CMemDBInterface::ReceiveBuffer(char *pBuffer, int nSize)
{
	char *pBuf;
	CString strErrorDesc;
	int nLeft, nCount, nRet = 0;

	//ڿһղ꣬ѭգֱչָֹͣ
	nCount = 0;
	pBuf = pBuffer;
	nLeft = nSize;
	memset(pBuffer, 0, nSize);
	while (nLeft > 0)
	{
		nRet = recv(m_skSocket, pBuf, nLeft, 0);
		if(nRet == 0)
		{
			//ӱر
			strErrorDesc = _T("ӱرˣֵΪ0");
			break;
		}
		else if (nRet == SOCKET_ERROR)
		{
			//ʧˣҪһЩ
			nRet = WSAGetLastError();
			if(nRet == WSAETIMEDOUT) //ʱҪٵȴһ
			{
				nCount++;
				if(nCount > 10)
				{
					strErrorDesc = _T("10ζֽճʱ");
					break;
				}
				Sleep(1);
			}
			else if(nRet == WSAECONNRESET) //ѾϿ
			{
				strErrorDesc = _T("ӱرˣֵΪ-1");
				return FALSE;
			}
			else //һ
			{
				nCount++;
				if(nCount > 10)
				{
					strErrorDesc.Format(_T("10ζֽմ󣬷ֵΪ-1WSAGetLastErrorķֵΪ0x%x"), nRet);
					break;
				}
				Sleep(1);
			}
		}
		else
		{
			nCount = 0;
			nLeft -= nRet;
			pBuf += nRet;
		}
	}

	if(nLeft != 0)
	{
		Trace(_T("ReceiveBuffer"), 0, 999011001, _T("շϢݳȲȷϢͷȣ%dʣ£%dδյ%s"), nSize, nLeft, strErrorDesc);
		return FALSE;
	}
	return TRUE;
}

HRESULT CMemDBInterface::TCPExecuteCommand(BSTR bstrCmd, LONG* plAffectCount)
{
	BYTE bToken;
	char* pSendData;
	CBaseBson stBson;
	int i, nRet, nSize, nCount;
	MF_SERVICE_TCP_COMMAND stTcpCmd;
	LPEXECUTEPLANBSON lpExecutePlan;

	try
	{
		CParseSql stParseSql;

		*plAffectCount= 0;
		lpExecutePlan = NULL;
		nRet = stParseSql.ParseSql(stBson, bstrCmd, lpExecutePlan);
		if(nRet != MF_OK)
		{
			return nRet;
		}
		else
		{
			CCriticalSectionPtr cs(&m_csInterFace);

			nCount = 0;
TCPExecuteCommand_Retry:
			bToken = CreateRandomFactor();
			nSize = sizeof(MF_SERVICE_TCP_COMMAND);
			memset(&stTcpCmd, 0, nSize);

			stTcpCmd.m_bVersion			= 1;
			stTcpCmd.m_bToken			= bToken;
			stTcpCmd.m_lDataLength		= lpExecutePlan->m_nTotalDataSize;

			pSendData = (char*)lpExecutePlan;
			stTcpCmd.m_bType = MF_INTERFACE_TCPCOMMAND_EXECUTECOMMAND;

			nRet = send(m_skSocket, (char *)&stTcpCmd , nSize, 0);
			if(nRet  != nSize)
			{
				//һӷ
				if(m_skSocket != INVALID_SOCKET)
				{
					closesocket(m_skSocket);
					m_skSocket = INVALID_SOCKET;
				}
				nRet = OpenDatabase(TRUE);
				if(nRet != MF_OK)
				{
					return E_FAIL;
				}
				nRet = send(m_skSocket, (char *)&stTcpCmd, nSize, 0);
				if(nRet != nSize)
				{
					Trace(_T("TCPExecuteCommand"), 0, 999010001, _T("ͨTCP˷ʧܣֵ%d룺%d"), nRet, WSAGetLastError());
					return E_FAIL;
				}
			}
			nRet = send(m_skSocket, pSendData, stTcpCmd.m_lDataLength, 0);
			if(nRet != stTcpCmd.m_lDataLength)
			{
				Trace(_T("TCPExecuteCommand"), 0, 999010001, _T("ͨTCP˷ʧܣֵ%d룺%d"), nRet, WSAGetLastError());
				return E_FAIL;
			}
			
			for(i = 0; i < stBson.GetBufferSectionNum(); i++)
			{
				pSendData = (char*)stBson.GetBufferSection(i);
				if(i == stBson.GetBufferSectionNum())
				{
					nRet = send(m_skSocket, pSendData, stBson.GetCurretDataSize(), 0);
				}
				else
				{
					nRet = send(m_skSocket, pSendData, lpExecutePlan->m_nDefaultSectionSize, 0);
				}
				if(nRet != stTcpCmd.m_lDataLength)
				{
					Trace(_T("TCPExecuteCommand"), 0, 999010001, _T("ͨTCP˷ʧܣֵ%d룺%d"), nRet, WSAGetLastError());
					return E_FAIL;
				}
			}

			//շ˷
			while(TRUE)
			{
				memset(&stTcpCmd, 0, sizeof(MF_SERVICE_TCP_COMMAND));
				nRet = recv(m_skSocket, (char *)&stTcpCmd, sizeof(MF_SERVICE_TCP_COMMAND), 0);
				if(nRet == 0)
				{
					//ӱر
					Trace(_T("TCPExecuteCommand"), 0, 999012001, _T("շʱӱرյĴ󣬷ֵ%d룺%d"), nRet, WSAGetLastError());
					return E_FAIL;
				}
				else if (nRet == SOCKET_ERROR)
				{
					//յSocket
					Trace(_T("TCPExecuteCommand"), 0, 999012002, _T("շʱSocket󣬷ֵ%d룺%d"), nRet, WSAGetLastError());
					return E_FAIL;
				}
				else if(nRet == sizeof(MF_SERVICE_TCP_COMMAND))
				{
					//жϷصǷȷ
					if(stTcpCmd.m_bType == MF_INTERFACE_TCPCOMMAND_TEST)
					{
						continue;
					}
					else if(stTcpCmd.m_bToken != bToken)
					{
						nCount++;
						if(nCount > 3)
						{
							return E_FAIL;
						}

						Sleep(1);
						ClearSocket();
						goto TCPExecuteCommand_Retry;
					}
					else
					{
						break;
					}
				}
				else
				{
					Trace(_T("TCPExecuteCommand"), 0, 999012003, _T("շʱֵģпǷ˵ȴʱͻԶTCPݣֵ%d룺%d"), nRet, WSAGetLastError());
					return E_FAIL;
				}
			}

			//жϷ
			if(stTcpCmd.m_bType == MF_INTERFACE_TCPCOMMAND_FAIL)
			{
				//Ϊʧܽ
				Trace(_T("TCPExecuteCommand"), 0, 999012004, _T("˷ִʧܣ룺%d"), stTcpCmd.m_lDigitalData);
				return E_FAIL;
			}
			else if(stTcpCmd.m_bType == MF_INTERFACE_TCPCOMMAND_OK)
			{
				if(stTcpCmd.m_lDataLength == 0)
				{
					*plAffectCount= stTcpCmd.m_lDigitalData;
					return S_OK;
				}
				else
				{
					//ݿ϶ȷֱ
					ClearSocket();
					Trace(_T("TCPExecuteCommand"), 0, 999012005, _T("˷صݲȷͻԶTCPݣΪ0ʵյ%d"), stTcpCmd.m_lDataLength);
					return E_FAIL;
				}
			}
			else
			{
				//ķֵ
				ClearSocket();
				Trace(_T("TCPExecuteCommand"), 0, 999012006, _T("˷صͲȷͻԶTCPݣյͣ%d"), stTcpCmd.m_bType);
				return E_FAIL;
			}
		}
	}
	catch(...)
	{
		Trace(_T("TCPExecuteCommand"), 0, 999012099, _T("ͨTCPִʱδ֪쳣룺%d"), GetLastError());
		return E_FAIL;
	}
	return S_OK;
}

HRESULT CMemDBInterface::TCPGetRecordset(BSTR bstrSql, _Recordset ** ppRs)
{
	HRESULT hr;
	BYTE bToken;
	char* pSendData;
	CBaseBson stBson;
	int i, nRet, nSize, nCount;
	LPEXECUTEPLANBSON lpExecutePlan;
	MF_SERVICE_TCP_COMMAND stTcpCmd;

	try
	{
		CParseSql stParseSql;
		lpExecutePlan = NULL;
		nRet = stParseSql.ParseSql(stBson, bstrSql, lpExecutePlan);
		if(nRet != MF_OK)
		{
			return E_FAIL;
		}
		else
		{
			CCriticalSectionPtr cs(&m_csInterFace);

			nCount = 0;
TCPGetRecordset_Retry:
			bToken = CreateRandomFactor();
			nSize = sizeof(MF_SERVICE_TCP_COMMAND);
			memset(&stTcpCmd, 0, nSize);
			stTcpCmd.m_bVersion			= 1;
			stTcpCmd.m_bToken			= bToken;
			stTcpCmd.m_lDataLength		= lpExecutePlan->m_nTotalDataSize;;

			pSendData	= (char*)lpExecutePlan;
			stTcpCmd.m_bType = MF_INTERFACE_TCPCOMMAND_GETRECORDSET;
			nRet = send(m_skSocket , (char *)&stTcpCmd , nSize, 0);
			if(nRet  != nSize)
			{
				//һӷ
				if(m_skSocket != INVALID_SOCKET)
				{
					closesocket(m_skSocket);
					m_skSocket = INVALID_SOCKET;
				}
				nRet = OpenDatabase(TRUE);
				if(nRet != MF_OK)
				{
					return E_FAIL;
				}
				nRet = send(m_skSocket , (char *)&stTcpCmd , nSize, 0);
				if(nRet  != nSize)
				{
					Trace(_T("TCPExecuteCommand"), 0, 999010001, _T("ͨTCP˷ʧܣֵ%d룺%d"), nRet, WSAGetLastError());
					return E_FAIL;
				}
			}
			nRet = send(m_skSocket , (char *)pSendData, stTcpCmd.m_lDataLength, 0);
			if(nRet  != stTcpCmd.m_lDataLength)
			{
				Trace(_T("TCPExecuteCommand"), 0, 999010001, _T("ͨTCP˷ʧܣֵ%d룺%d"), nRet, WSAGetLastError());
				return E_FAIL;
			}
			
			for(i = 0; i < stBson.GetBufferSectionNum(); i++)
			{
				pSendData = (char*)stBson.GetBufferSection(i);
				if(i == stBson.GetBufferSectionNum())
				{
					nRet = send(m_skSocket, pSendData, stBson.GetCurretDataSize(), 0);
				}
				else
				{
					nRet = send(m_skSocket, pSendData, lpExecutePlan->m_nDefaultSectionSize, 0);
				}
				if(nRet != stTcpCmd.m_lDataLength)
				{
					Trace(_T("TCPExecuteCommand"), 0, 999010001, _T("ͨTCP˷ʧܣֵ%d룺%d"), nRet, WSAGetLastError());
					return E_FAIL;
				}
			}
			//շ˷
			while(TRUE)
			{
				memset(&stTcpCmd, 0, sizeof(MF_SERVICE_TCP_COMMAND));
				nRet = recv(m_skSocket, (char *)&stTcpCmd, sizeof(MF_SERVICE_TCP_COMMAND), 0);
				if(nRet == 0)
				{
					//ӱر
					Trace(_T("TCPGetRecordset"), 0, 999013001, _T("շʱӱرյĴ󣬷ֵ%d룺%d"), nRet, WSAGetLastError());
					return E_FAIL;
				}
				else if (nRet == SOCKET_ERROR)
				{
					//յSocket
					Trace(_T("TCPGetRecordset"), 0, 999013002, _T("շʱSocket󣬷ֵ%d룺%d"), nRet, WSAGetLastError());
					return E_FAIL;
				}
				else if(nRet == sizeof(MF_SERVICE_TCP_COMMAND))
				{
					//жϷصǷȷ
					if(stTcpCmd.m_bType == MF_INTERFACE_TCPCOMMAND_TEST)
					{
						continue;
					}
					else if(stTcpCmd.m_bToken != bToken)
					{
						nCount++;
						if(nCount > 3)
						{
							return E_FAIL;
						}

						Sleep(1);
						ClearSocket();
						goto TCPGetRecordset_Retry;
					}
					else
					{
						break;
					}
				}
				else
				{
					Trace(_T("TCPGetRecordset"), 0, 999013003, _T("շʱֵģпǷ˵ȴʱͻԶTCPݣֵ%d룺%d"), nRet, WSAGetLastError());
					return E_FAIL;
				}
			}

			//жϷ
			if(stTcpCmd.m_bType == MF_INTERFACE_TCPCOMMAND_FAIL)
			{
				//Ϊʧܽ
				Trace(_T("TCPGetRecordset"), 0, 999013004, _T("˷ִʧܣ룺%d"), stTcpCmd.m_lDigitalData);
				return E_FAIL;
			}
			else if(stTcpCmd.m_bType == MF_INTERFACE_TCPCOMMAND_OK)
			{
				//ΪɹҪպ
				if(stTcpCmd.m_lDataLength == 0)
				{
					Trace0(_T("TCPGetRecordset"), 0, 999013005, _T("˷صݲȷΪ0ʵյ0"));
					return E_FAIL;
				}
				else
				{
					char* lpBuffer;
					CComBSTR bstrRs;
					std::string strOutput;
					CBaseMemoryAnalysis stAnalysis;
					EXECUTESTATISTICSINFO stExecuteInfo;
					memset(&stExecuteInfo, 0, sizeof(EXECUTESTATISTICSINFO));
					QueryPerformanceFrequency(&stExecuteInfo.m_liFrequence);
					QueryPerformanceCounter(&stExecuteInfo.m_liStart);
					lpBuffer = new char[stTcpCmd.m_lDataLength];
					if(lpBuffer == NULL)
					{
						Trace(_T("TCPGetRecordset"), 0, 999013006, _T("BSONݵڴʧܣڴСΪ%d룺%d"), stTcpCmd.m_lDataLength, GetLastError());
						ClearSocket();
						return E_FAIL;
					}
					if(!ReceiveBuffer(lpBuffer, stTcpCmd.m_lDataLength))
					{
						delete [] lpBuffer;
						return E_FAIL;
					}

					//ȹXML
					nRet = stAnalysis.BuildXML(strOutput, (LPBYTE)lpBuffer, &stExecuteInfo);
					ConvertBSTR(strOutput.c_str(), bstrRs);

					//תһRecorset
					hr = ConvertToRecordset(bstrRs, ppRs);
					delete [] lpBuffer;
					lpBuffer = NULL;
					return hr;
				}
			}
			else
			{
				//ķֵ
				ClearSocket();
				Trace(_T("TCPGetRecordset"), 0, 999013099, _T("˷صͲȷͻԶTCPݣյͣ%d"), stTcpCmd.m_bType);
				return E_FAIL;
			}
		}
	}
	catch(...)
	{
		Trace(_T("TCPGetRecordset"), 0, 999013099, _T("ͨTCPִмʱδ֪쳣룺%d"), GetLastError());
		return E_FAIL;
	}
	return S_OK;
}

HRESULT CMemDBInterface::TCPUpdateRecordset(_Recordset * pRs)
{
	HRESULT hr;
	BYTE bToken;
	char* pSendData;
	CComBSTR btrXml;
	CBaseBson stBson;
	int i, nRet, nSize, nCount;
	MF_SERVICE_TCP_COMMAND stTcpCmd;
	LPEXECUTEPLANBSON lpExecutePlan;

	try
	{
		CBaseMemoryAnalysis stAnalysis;
		hr = ConvertFromRecordset(pRs, btrXml);
		if(FAILED(hr))
		{
			return E_FAIL;
		}
		lpExecutePlan = NULL;
		nRet = stAnalysis.ParseXML(stBson, btrXml, lpExecutePlan);
		if(nRet != MF_OK)
		{
			return E_FAIL;
		}
		else
		{
			CCriticalSectionPtr cs(&m_csInterFace);
			nCount = 0;
TCPUpdateRecordset_Retry:
			bToken = CreateRandomFactor();
			//
			nSize = sizeof(MF_SERVICE_TCP_COMMAND);
			memset(&stTcpCmd, 0, nSize);
			stTcpCmd.m_bVersion			= 1;
			stTcpCmd.m_lDataLength		= lpExecutePlan->m_nTotalDataSize;
			stTcpCmd.m_bToken			= bToken;
			stTcpCmd.m_bType			= MF_INTERFACE_TCPCOMMAND_UPDATERECORDSET;

			nRet = send(m_skSocket , (char *)&stTcpCmd , nSize, 0);
			if(nRet  != nSize)
			{
				//һӷ
				if(m_skSocket != INVALID_SOCKET)
				{
					closesocket(m_skSocket);
					m_skSocket = INVALID_SOCKET;
				}
				nRet = OpenDatabase(TRUE);
				if(nRet != MF_OK)
				{
					return E_FAIL;
				}
				nRet = send(m_skSocket , (char *)&stTcpCmd , nSize, 0);
				if(nRet  != nSize)
				{
					Trace(_T("TCPExecuteCommand"), 0, 999010001, _T("ͨTCP˷ʧܣֵ%d룺%d"), nRet, WSAGetLastError());
					return E_FAIL;
				}
			}
			nRet = send(m_skSocket , (char *)lpExecutePlan, lpExecutePlan->m_nClientBsonDataSize, 0);
			if(nRet  !=  lpExecutePlan->m_nClientBsonDataSize)
			{
				Trace(_T("TCPExecuteCommand"), 0, 999010001, _T("ͨTCP˷ʧܣֵ%d룺%d"), nRet, WSAGetLastError());
				return E_FAIL;
			}
			
			for(i = 0; i < stBson.GetBufferSectionNum(); i++)
			{
				pSendData = (char*)stBson.GetBufferSection(i+1);
				if(i == stBson.GetBufferSectionNum() - 1)
				{
					nSize = stBson.GetCurretDataSize();
				}
				else
				{
					nSize = lpExecutePlan->m_nDefaultSectionSize;
				}

				nRet = send(m_skSocket, pSendData, nSize, 0);
				if(nRet != nSize)
				{
					Trace(_T("TCPExecuteCommand"), 0, 999010001, _T("ͨTCP˷ʧܣֵ%d룺%d"), nRet, WSAGetLastError());
					return E_FAIL;
				}
			}
			//շ˷
			while(TRUE)
			{
				memset(&stTcpCmd, 0, sizeof(MF_SERVICE_TCP_COMMAND));
				nRet = recv(m_skSocket, (char *)&stTcpCmd, sizeof(MF_SERVICE_TCP_COMMAND), 0);
				if(nRet == 0)
				{
					//ӱر
					Trace(_T("TCPUpdateRecordset"), 0, 999014001, _T("շʱӱرյĴ󣬷ֵ%d룺%d"), nRet, WSAGetLastError());
					return E_FAIL;
				}
				else if (nRet == SOCKET_ERROR)
				{
					//յSocket
					Trace(_T("TCPUpdateRecordset"), 0, 999014002, _T("շʱSocket󣬷ֵ%d룺%d"), nRet, WSAGetLastError());
					return E_FAIL;
				}
				else if(nRet == sizeof(MF_SERVICE_TCP_COMMAND))
				{
					//жϷصǷȷ
					if(stTcpCmd.m_bType == MF_INTERFACE_TCPCOMMAND_TEST)
					{
						continue;
					}
					else if(stTcpCmd.m_bToken != bToken)
					{
						nCount++;
						if(nCount > 3)
						{
							return E_FAIL;
						}

						Sleep(1);
						ClearSocket();
						goto TCPUpdateRecordset_Retry;
					}
					else
					{
						break;
					}
				}
				else
				{
					Trace(_T("TCPUpdateRecordset"), 0, 999014003, _T("շʱֵģпǷ˵ȴʱͻԶTCPݣֵ%d룺%d"), nRet, WSAGetLastError());
					return E_FAIL;
				}
			}

			//жϷ
			if(stTcpCmd.m_bType == MF_INTERFACE_TCPCOMMAND_FAIL)
			{
				//Ϊʧܽ
				Trace(_T("TCPUpdateRecordset"), 0, 999013004, _T("˷ִʧܣ룺%d"), stTcpCmd.m_lDigitalData);
				return E_FAIL;
			}
			else if(stTcpCmd.m_bType == MF_INTERFACE_TCPCOMMAND_OK)
			{
				//Ϊɹ
				if(stTcpCmd.m_lDataLength == 0)
				{
					return S_OK;
				}
				else
				{
					//ݿ϶ȷֱ
					ClearSocket();
					Trace(_T("TCPUpdateRecordset"), 0, 999014005, _T("˷صݲȷͻԶTCPݣΪ0ʵյ%d"), stTcpCmd.m_lDataLength);
					return E_FAIL;
				}
			}
			else
			{
				//ķֵ
				ClearSocket();
				Trace(_T("TCPUpdateRecordset"), 0, 999014006, _T("˷صͲȷͻԶTCPݣյͣ%d"), stTcpCmd.m_bType);
				return E_FAIL;
			}
		}
	}
	catch(...)
	{
		Trace(_T("TCPUpdateRecordset"), 0, 999014099, _T("ͨTCPִ޸ʱδ֪쳣룺%d"), GetLastError());
		return E_FAIL;
	}
	return S_OK;
}


STDMETHODIMP CMemDBInterface::Open(BSTR bstrServer, BSTR bstrUser, BSTR bstrPassword)
{
	char lpszServerName[64], lpszUserName[64], lpszPassword[64];
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	m_lInterfaceType   = 0;
	Close();

	memset(lpszServerName, 0, sizeof(lpszServerName));
	memset(lpszUserName, 0, sizeof(lpszUserName));
	memset(lpszPassword, 0, sizeof(lpszPassword));
	WideCharToMultiByte(CP_UTF8, NULL, bstrServer,  ::SysStringLen(bstrServer), lpszServerName, 64, NULL, NULL);
	WideCharToMultiByte(CP_UTF8, NULL, bstrUser,  ::SysStringLen(bstrUser), lpszUserName, 64, NULL, NULL);
	WideCharToMultiByte(CP_UTF8, NULL, bstrPassword,  ::SysStringLen(bstrPassword), lpszPassword, 64, NULL, NULL);
	return OpenCore(lpszServerName, lpszUserName, lpszPassword);
}

HRESULT CMemDBInterface::TCPStartTransactionLogic(BSTR bstrTransactionData, LONG* plTransactionID)
{
	BYTE bToken;
	char *lpData;
	int nSize, nRet, nCount;
	MF_SERVICE_TCP_COMMAND stTcpCmd;
	CCriticalSectionPtr cs(&m_csInterFace);
	try
	{
		nCount = 0;
TCPStartTransactionLogic_Retry:
		bToken = CreateRandomFactor();
		nSize = sizeof(MF_SERVICE_TCP_COMMAND);
		memset(&stTcpCmd, 0, nSize);
		stTcpCmd.m_bVersion			= 1;
		stTcpCmd.m_bType			= MF_INTERFACE_TCPCOMMAND_STARTTRANSACTIONLOGIC;
		stTcpCmd.m_lDataLength		= 0;
		stTcpCmd.m_bToken			= bToken;
		lpData = ConvertToUTF8(bstrTransactionData);
		if(strlen(lpData) > 99)
		{
			Trace(_T("TCPStartTransactionLogic"), 0, 999040000, _T("ʼַ̫ʧܣַ%s"), bstrTransactionData);
			delete [] lpData;
			return E_FAIL;
		}
		strcpy_s(stTcpCmd.m_lpszParam, 100, lpData);
		delete [] lpData;

		nRet = send(m_skSocket , (char *)&stTcpCmd , nSize, 0);
		if(nRet  != nSize)
		{
			//һӷ
			if(m_skSocket != INVALID_SOCKET)
			{
				closesocket(m_skSocket);
				m_skSocket = INVALID_SOCKET;
			}
			nRet = OpenDatabase(TRUE);
			if(nRet != MF_OK)
			{
				return E_FAIL;
			}
			nRet = send(m_skSocket , (char *)&stTcpCmd , nSize, 0);
			if(nRet  != nSize)
			{
				Trace(_T("TCPStartTransactionLogic"), 0, 999040001, _T("ͨTCP˷ʧܣֵ%d룺%d"), nRet, WSAGetLastError());
				return E_FAIL;
			}
		}

		//շ˷
		while(TRUE)
		{
			memset(&stTcpCmd, 0, sizeof(MF_SERVICE_TCP_COMMAND));
			nRet = recv(m_skSocket, (char *)&stTcpCmd, sizeof(MF_SERVICE_TCP_COMMAND), 0);
			if(nRet == 0)
			{
				//ӱر
				Trace(_T("TCPStartTransactionLogic"), 0, 999040001, _T("շʱӱرյĴ󣬷ֵ%d룺%d"), nRet, WSAGetLastError());
				return E_FAIL;
			}
			else if (nRet == SOCKET_ERROR)
			{
				//յSocket
				Trace(_T("TCPStartTransactionLogic"), 0, 999040002, _T("շʱSocket󣬷ֵ%d룺%d"), nRet, WSAGetLastError());
				return E_FAIL;
			}
			else if(nRet == sizeof(MF_SERVICE_TCP_COMMAND))
			{
				//жϷصǷȷ
				if(stTcpCmd.m_bType == MF_INTERFACE_TCPCOMMAND_TEST)
				{
					continue;
				}
				else if(stTcpCmd.m_bToken != bToken)
				{
					nCount++;
					if(nCount > 3)
					{
						return E_FAIL;
					}

					Sleep(1);
					ClearSocket();
					goto TCPStartTransactionLogic_Retry;
				}
				else
				{
					break;
				}
			}
			else
			{
				Trace(_T("TCPStartTransactionLogic"), 0, 999040003, _T("շʱֵģпǷ˵ȴʱͻԶTCPݣֵ%d룺%d"), nRet, WSAGetLastError());
				return E_FAIL;
			}
		}

		//жϷ
		if(stTcpCmd.m_bType == MF_INTERFACE_TCPCOMMAND_FAIL)
		{
			//Ϊʧܽ
			Trace(_T("TCPStartTransactionLogic"), 0, 999040004, _T("˷ִʧܣ룺%d"), stTcpCmd.m_lDigitalData);
			return E_FAIL;
		}
		else if(stTcpCmd.m_bType == MF_INTERFACE_TCPCOMMAND_OK)
		{
			//Ϊɹ
			if(stTcpCmd.m_lDataLength == 0)
			{
				*plTransactionID = stTcpCmd.m_lDigitalData;
				return S_OK;
			}
			else
			{
				//ݿ϶ȷֱ
				ClearSocket();
				Trace(_T("TCPStartTransactionLogic"), 0, 999040005, _T("˷صݲȷͻԶTCPݣΪ0ʵյ%d"), stTcpCmd.m_lDataLength);
				return E_FAIL;
			}
		}
		else
		{
			//ķֵ
			ClearSocket();
			Trace(_T("TCPStartTransactionLogic"), 0, 999040006, _T("˷صͲȷͻԶTCPݣյͣ%d"), stTcpCmd.m_bType);
			return E_FAIL;
		}
	}
	catch(...)
	{
		Trace(_T("TCPStartTransactionLogic"), 0, 999040099, _T("ͨTCPִпʱδ֪쳣룺%d"), GetLastError());
		return E_FAIL;
	}
	return S_OK;
}

HRESULT CMemDBInterface::StartTransactionLogicCore(BSTR bstrTransactionData, LONG* plTransactionID)
{
	HRESULT hr;
	__try
	{
		if(!m_bOpenFlag)
		{
			return E_FAIL;
		}
		//ʹTCPʽӷ
		hr = TCPStartTransactionLogic(bstrTransactionData, plTransactionID);
	}
	__except(MyExceptionHandler(GetExceptionCode(), GetExceptionInformation()))
	{
		Trace(_T("GetRecordset"), 0, 999042099, _T("ݿִ쳣ֵ0X%XbstrTransactionData%sӿͣ%d"), hr, bstrTransactionData, m_lInterfaceType);
		return E_FAIL;
	}
	return hr;
}

STDMETHODIMP CMemDBInterface::StartTransactionLogic(BSTR bstrTransactionData, LONG* plTransactionID)
{
	HRESULT hr;
	DWORD dwTime;
	LARGE_INTEGER liFrequence, liStart, liEnd;
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	QueryPerformanceFrequency(&liFrequence);
	QueryPerformanceCounter(&liStart);
	hr = StartTransactionLogicCore(bstrTransactionData, plTransactionID);
	QueryPerformanceCounter(&liEnd);
	if(m_lWriteEfficiencyLog == 1)
	{
		dwTime = (DWORD)(1000000 * (liEnd.QuadPart - liStart.QuadPart) / liFrequence.QuadPart); //㵽΢
		Trace(_T("ExecuteCommand"), 0, 999043099, _T("ʼִнʱ%d΢룬ֵ0X%XID%dbstrTransactionData%sӿͣ%d"), dwTime, hr, *plTransactionID, bstrTransactionData, m_lInterfaceType);
	}

	return hr;
}

HRESULT CMemDBInterface::TCPStopTransactionLogic(LONG lTransactionID)
{
	BYTE bToken;
	int nSize, nRet, nCount;
	MF_SERVICE_TCP_COMMAND stTcpCmd;
	CCriticalSectionPtr cs(&m_csInterFace);
	try
	{
		nCount = 0;
TCPStopTransactionLogic_Retry:
		bToken = CreateRandomFactor();
		nSize = sizeof(MF_SERVICE_TCP_COMMAND);
		memset(&stTcpCmd, 0, nSize);
		stTcpCmd.m_bVersion			= 1;
		stTcpCmd.m_bType			= MF_INTERFACE_TCPCOMMAND_STOPTRANSACTIONLOGIC;
		stTcpCmd.m_lDigitalData		= lTransactionID;
		stTcpCmd.m_lDataLength		= 0;
		stTcpCmd.m_bToken			= bToken;

		nRet = send(m_skSocket , (char *)&stTcpCmd , nSize, 0);
		if(nRet  != nSize)
		{
			//һӷ
			if(m_skSocket != INVALID_SOCKET)
			{
				closesocket(m_skSocket);
				m_skSocket = INVALID_SOCKET;
			}
			nRet = OpenDatabase(TRUE);
			if(nRet != MF_OK)
			{
				return E_FAIL;
			}
			nRet = send(m_skSocket , (char *)&stTcpCmd , nSize, 0);
			if(nRet  != nSize)
			{
				Trace(_T("TCPStopTransactionLogic"), 0, 999044001, _T("ͨTCP˷ʧܣֵ%d룺%d"), nRet, WSAGetLastError());
				return E_FAIL;
			}
		}

		//շ˷
		while(TRUE)
		{
			memset(&stTcpCmd, 0, sizeof(MF_SERVICE_TCP_COMMAND));
			nRet = recv(m_skSocket, (char *)&stTcpCmd, sizeof(MF_SERVICE_TCP_COMMAND), 0);
			if(nRet == 0)
			{
				//ӱر
				Trace(_T("TCPStopTransactionLogic"), 0, 999044001, _T("շʱӱرյĴ󣬷ֵ%d룺%d"), nRet, WSAGetLastError());
				return E_FAIL;
			}
			else if (nRet == SOCKET_ERROR)
			{
				//յSocket
				Trace(_T("TCPStopTransactionLogic"), 0, 999044002, _T("շʱSocket󣬷ֵ%d룺%d"), nRet, WSAGetLastError());
				return E_FAIL;
			}
			else if(nRet == sizeof(MF_SERVICE_TCP_COMMAND))
			{
				//жϷصǷȷ
				if(stTcpCmd.m_bType == MF_INTERFACE_TCPCOMMAND_TEST)
				{
					continue;
				}
				else if(stTcpCmd.m_bToken != bToken)
				{
					nCount++;
					if(nCount > 3)
					{
						return E_FAIL;
					}

					Sleep(1);
					ClearSocket();
					goto TCPStopTransactionLogic_Retry;
				}
				else
				{
					break;
				}
			}
			else
			{
				Trace(_T("TCPStopTransactionLogic"), 0, 999044003, _T("շʱֵģпǷ˵ȴʱͻԶTCPݣֵ%d룺%d"), nRet, WSAGetLastError());
				return E_FAIL;
			}
		}

		//жϷ
		if(stTcpCmd.m_bType == MF_INTERFACE_TCPCOMMAND_FAIL)
		{
			//Ϊʧܽ
			Trace(_T("TCPStopTransactionLogic"), 0, 999044004, _T("˷ִʧܣ룺%d"), stTcpCmd.m_lDigitalData);
			return E_FAIL;
		}
		else if(stTcpCmd.m_bType == MF_INTERFACE_TCPCOMMAND_OK)
		{
			//Ϊɹ
			if(stTcpCmd.m_lDataLength == 0)
			{
				return S_OK;
			}
			else
			{
				//ݿ϶ȷֱ
				ClearSocket();
				Trace(_T("TCPStopTransactionLogic"), 0, 999044005, _T("˷صݲȷͻԶTCPݣΪ0ʵյ%d"), stTcpCmd.m_lDataLength);
				return E_FAIL;
			}
		}
		else
		{
			//ķֵ
			ClearSocket();
			Trace(_T("TCPStopTransactionLogic"), 0, 999044006, _T("˷صͲȷͻԶTCPݣյͣ%d"), stTcpCmd.m_bType);
			return E_FAIL;
		}
	}
	catch(...)
	{
		Trace(_T("TCPStopTransactionLogic"), 0, 999044099, _T("ͨTCPִпʱδ֪쳣룺%d"), GetLastError());
		return E_FAIL;
	}
	return S_OK;
}

HRESULT CMemDBInterface::StopTransactionLogicCore(LONG lTransactionID)
{
	HRESULT hr;
	__try
	{
		if(!m_bOpenFlag)
		{
			return E_FAIL;
		}
		//ʹTCPʽӷ
		hr = TCPStopTransactionLogic(lTransactionID);
	}
	__except(MyExceptionHandler(GetExceptionCode(), GetExceptionInformation()))
	{
		Trace(_T("StopTransactionLogicCore"), 0, 999046099, _T("ִ쳣ֵ0X%XlTransactionID%dӿͣ%d"), hr, lTransactionID, m_lInterfaceType);
		return E_FAIL;
	}
	return hr;
}


STDMETHODIMP CMemDBInterface::StopTransactionLogic(LONG lTransactionID)
{
	HRESULT hr;
	DWORD dwTime;
	LARGE_INTEGER liFrequence, liStart, liEnd;
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	QueryPerformanceFrequency(&liFrequence);
	QueryPerformanceCounter(&liStart);
	hr = StopTransactionLogicCore(lTransactionID);
	QueryPerformanceCounter(&liEnd);
	if(m_lWriteEfficiencyLog == 1)
	{
		dwTime = (DWORD)(1000000 * (liEnd.QuadPart - liStart.QuadPart) / liFrequence.QuadPart); //㵽΢
		Trace(_T("ExecuteCommand"), 0, 999047099, _T("ִнʱ%d΢룬ֵ0X%XlTransactionID%dӿͣ%d"), dwTime, hr, lTransactionID, m_lInterfaceType);
	}

	return hr;
}

//رݿӺ
void CMemDBInterface::Close()
{
	int nRetValue = 0;
	if(m_skSocket != INVALID_SOCKET)
	{
		UserLogOut(nRetValue);
		closesocket(m_skSocket);
		m_skSocket = INVALID_SOCKET;
	}
}

BOOL CMemDBInterface::IsCharacter(char c)
{
	if ((c >= 48 && c <= 57) || (c>= 65 && c <= 90) || (c >= 97 && c <= 122) || c == '_' || c == '.' ||  c == '-')		//֣дĸСдĸ»ߣС㣬
	{
		return TRUE;
	} 
	else
	{
		return FALSE;
	}
}

BOOL CMemDBInterface::ParseConnectConfig(char *lpServerName, FILE * fp, STRUCT_CONNECT_CONFIG &stConfig)
{
	BOOL bValue;
	char *lpBuffer;
	STRUCT_ADDRESS stAddr;
	STRUCT_CONNECT_CONFIG st;
	int i, j, nIndex, nLeft, nRight, nEqualSign, nLineLength, nStrLength;
	char arrayLine[1024];

	lpBuffer = NULL;
	j = 0;
	nIndex = 0;
	nLeft = 0;
	nRight = 0;
	nEqualSign = 0;
	bValue = FALSE;
	memset(&st.m_cSobeyDB, 0, sizeof(st.m_cSobeyDB));
	memset(&st.m_cDatabaseName, 0, sizeof(st.m_cDatabaseName));
	st.m_vecAddress.clear();
	memset(&stAddr, 0, sizeof(stAddr));
	st.m_vecAddress.push_back(stAddr);
	while(!feof(fp))
	{
		memset(arrayLine, 0, sizeof(arrayLine));
		if(fgets(arrayLine, 1000, fp))		//ȡһ
		{
			nLineLength = strlen(arrayLine);
			i = 0;
			while (i < nLineLength)
			{
				if (arrayLine[i] == ' ' || arrayLine[i] == '\t')
				{
					i++;
					continue;
				}
				if (arrayLine[i] == '#' || arrayLine[i] == '\n' || arrayLine[i] == '\r')		//ע
				{
					break;
				}
				if (arrayLine[i] == '(')
				{
					nLeft++;
					i++;
					continue;
				}
				if (arrayLine[i] == ')')
				{
					nRight++;
					i++;
					if (nRight == 9)
					{
						if (nLeft == 9 && nEqualSign == 10 && nIndex == 10)		//ʼһò
						{
							if(stricmp(st.m_cSobeyDB, lpServerName) == 0)
							{
								memcpy(stConfig.m_cSobeyDB, st.m_cSobeyDB, sizeof(st.m_cSobeyDB));
								memcpy(stConfig.m_cDatabaseName, st.m_cDatabaseName, sizeof(st.m_cDatabaseName));
								for(j = 0;j < st.m_vecAddress.size();j++)
								{
									stConfig.m_vecAddress.push_back(st.m_vecAddress[j]);
								}
								return TRUE;
							}
							j = 0;
							nIndex = 0;
							nLeft = 0;
							nRight = 0;
							nEqualSign = 0;
							bValue = FALSE;
							memset(&st.m_cSobeyDB, 0, sizeof(st.m_cSobeyDB));
							memset(&st.m_cDatabaseName, 0, sizeof(st.m_cDatabaseName));
							st.m_vecAddress.clear();
							memset(&stAddr, 0, sizeof(stAddr));
							st.m_vecAddress.push_back(stAddr);
						}
						else
						{
							return FALSE;
						}
					}
					continue;
				}
				if (arrayLine[i] == '=')
				{
					nEqualSign++;
					i++;
					continue;
				}

				if (!IsCharacter(arrayLine[i]))
				{
					return FALSE;
				}

				if (0 == nIndex)			//ʶ
				{
					if (nLeft == nIndex && nRight == 0 && nEqualSign == nIndex)
					{
						nStrLength = 0;
						lpBuffer = arrayLine+i;
						while (i < nLineLength)
						{
							if (IsCharacter(arrayLine[i]))
							{
								i++;
								nStrLength++;
								continue;
							}
							memcpy(st.m_cSobeyDB, lpBuffer, nStrLength);
							nIndex++;
							break;
						}
						if (nIndex != 1)
						{
							return FALSE;
						}
					} 
					else
					{
						return FALSE;
					}
				}
				else if (1 == nIndex)			//DESCRIPTION
				{
					if (nLeft == nIndex && nRight == 0 && nEqualSign == nIndex)
					{
						nStrLength = 11;
						lpBuffer = arrayLine+i;
						if (i + nStrLength < nLineLength)
						{
#ifdef WIN32
							if (_strnicmp(lpBuffer, "DESCRIPTION", nStrLength) == 0)
#else
							if (strncasecmp(lpBuffer, "DESCRIPTION", nStrLength) == 0)
#endif
							{
								nIndex++;
								i = i+nStrLength;
							} 
							else
							{
								return FALSE;
							}
						}
						else
						{
							return FALSE;
						}
					}
					else
					{
						return FALSE;
					}
				}
				else if (2 == nIndex)			//ADDRESS_LIST
				{
					if (nLeft == nIndex && nRight == 0 && nEqualSign == nIndex)
					{
						nStrLength = 12;
						lpBuffer = arrayLine+i;
						if (i + nStrLength < nLineLength)
						{
#ifdef WIN32
							if (_strnicmp(lpBuffer, "ADDRESS_LIST", nStrLength) == 0)
#else
							if (strncasecmp(lpBuffer, "ADDRESS_LIST", nStrLength) == 0)
#endif
							{
								nIndex++;
								i = i+nStrLength;
							} 
							else
							{
								return FALSE;
							}
						}
						else
						{
							return FALSE;
						}
					}
					else
					{
						return FALSE;
					}
				}
				else if (3 == nIndex)			//ADDRESS
				{
					if (nLeft == nIndex && nRight == 0 && nEqualSign == nIndex)
					{
						nStrLength = 7;
						lpBuffer = arrayLine+i;
						if (i + nStrLength < nLineLength)
						{
#ifdef WIN32
							if (_strnicmp(lpBuffer, "ADDRESS", nStrLength) == 0)
#else
							if (strncasecmp(lpBuffer, "ADDRESS", nStrLength) == 0)
#endif
							{
								nIndex++;
								i = i+nStrLength;
							} 
							else
							{
								return FALSE;
							}
						}
						else
						{
							return FALSE;
						}
					}
					else
					{
						return FALSE;
					}
				}
				else if (4 == nIndex)			//PROTOCOL
				{
					if (bValue == FALSE)
					{
						if (nLeft == nIndex && nRight == 0 && nEqualSign == nIndex)
						{
							nStrLength = 8;
							lpBuffer = arrayLine+i;
							if (i + nStrLength < nLineLength)
							{
#ifdef WIN32
								if (_strnicmp(lpBuffer, "PROTOCOL", nStrLength) == 0)
#else
								if (strncasecmp(lpBuffer, "PROTOCOL", nStrLength) == 0)
#endif
								{
									i = i+nStrLength;
									bValue = TRUE;
								} 
								else
								{
									return FALSE;
								}
							}
							else
							{
								return FALSE;
							}
						}
						else
						{
							return FALSE;
						}
					} 
					else
					{
						if (nLeft == nIndex && nRight == 0 && nEqualSign == nIndex+1)
						{
							nStrLength = 0;
							lpBuffer = arrayLine+i;
							while (i < nLineLength)
							{
								if (IsCharacter(arrayLine[i]))
								{
									i++;
									nStrLength++;
									continue;
								}
								memcpy(st.m_vecAddress[j].m_cProtocol, lpBuffer, nStrLength);
								nIndex++;
								bValue = FALSE;
								break;
							}
							if (nIndex != 5)
							{
								return FALSE;
							}
						} 
						else
						{
							return FALSE;
						}
					}
				}
				else if (5 == nIndex)			//HOST
				{
					if (bValue == FALSE)
					{
						if (nLeft == nIndex && nRight == 1 && nEqualSign == nIndex)
						{
							nStrLength = 4;
							lpBuffer = arrayLine+i;
							if (i + nStrLength < nLineLength)
							{
#ifdef WIN32
								if (_strnicmp(lpBuffer, "HOST", nStrLength) == 0)
#else
								if (strncasecmp(lpBuffer, "HOST", nStrLength) == 0)
#endif
								{
									i = i+nStrLength;
									bValue = TRUE;
								} 
								else
								{
									return FALSE;
								}
							}
							else
							{
								return FALSE;
							}
						}
						else
						{
							return FALSE;
						}
					} 
					else
					{
						if (nLeft == nIndex && nRight == 1 && nEqualSign == nIndex+1)
						{
							nStrLength = 0;
							lpBuffer = arrayLine+i;
							while (i < nLineLength)
							{
								if (IsCharacter(arrayLine[i]))
								{
									i++;
									nStrLength++;
									continue;
								}
								memcpy(st.m_vecAddress[j].m_cHost, lpBuffer, nStrLength);
								nIndex++;
								bValue = FALSE;
								break;
							}
							if (nIndex != 6)
							{
								return FALSE;
							}
						} 
						else
						{
							return FALSE;
						}
					}
				}
				else if (6 == nIndex)			//PORT
				{
					if (bValue == FALSE)
					{
						if (nLeft == nIndex && nRight == 2 && nEqualSign == nIndex)
						{
							nStrLength = 4;
							lpBuffer = arrayLine+i;
							if (i + nStrLength < nLineLength)
							{
#ifdef WIN32
								if (_strnicmp(lpBuffer, "PORT", nStrLength) == 0)
#else
								if (strncasecmp(lpBuffer, "PORT", nStrLength) == 0)
#endif
								{
									i = i+nStrLength;
									bValue = TRUE;
								} 
								else
								{
									return FALSE;
								}
							}
							else
							{
								return FALSE;
							}
						}
						else
						{
							return FALSE;
						}
					} 
					else
					{
						if (nLeft == nIndex && nRight == 2 && nEqualSign == nIndex+1)
						{
							nStrLength = 0;
							lpBuffer = arrayLine+i;
							while (i < nLineLength)
							{
								if (IsCharacter(arrayLine[i]))
								{
									i++;
									nStrLength++;
									continue;
								}
								memcpy(st.m_vecAddress[j].m_cPort, lpBuffer, nStrLength);
								nIndex++;
								bValue = FALSE;
								break;
							}
							if (nIndex != 7)
							{
								return FALSE;
							}
						} 
						else
						{
							return FALSE;
						}
					}
				}
				else if (7 == nIndex)			//WriteLog
				{
					if (bValue == FALSE)
					{
						if (nLeft == nIndex && nRight == 3 && nEqualSign == nIndex)
						{
							nStrLength = 8;
							lpBuffer = arrayLine+i;
							if (i + nStrLength < nLineLength)
							{
#ifdef WIN32
								if (_strnicmp(lpBuffer, "WriteLog", nStrLength) == 0)
#else
								if (strncasecmp(lpBuffer, "WriteLog", nStrLength) == 0)
#endif
								{
									i = i+nStrLength;
									bValue = TRUE;
								} 
								else
								{
									return FALSE;
								}
							}
							else
							{
								return FALSE;
							}
						}
						else
						{
							return FALSE;
						}
					} 
					else
					{
						if (nLeft == nIndex && nRight == 3 && nEqualSign == nIndex+1)
						{
							nStrLength = 0;
							lpBuffer = arrayLine+i;
							while (i < nLineLength)
							{
								if (IsCharacter(arrayLine[i]))
								{
									i++;
									nStrLength++;
									continue;
								}
								memcpy(st.m_vecAddress[j].m_cWriteLog, lpBuffer, nStrLength);
								nIndex++;
								bValue = FALSE;
								break;
							}
							if (nIndex != 8)
							{
								return FALSE;
							}
						} 
						else
						{
							return FALSE;
						}
					}
				}
				else if (8 == nIndex)			//CONNECT_DATA
				{
					if (nLeft == nIndex && nRight == 6 && nEqualSign == nIndex)				//CONNECT_DATA
					{
						nStrLength = 12;
						lpBuffer = arrayLine+i;
						if (i + nStrLength < nLineLength)
						{
#ifdef WIN32
							if (_strnicmp(lpBuffer, "CONNECT_DATA", nStrLength) == 0)
#else
							if (strncasecmp(lpBuffer, "CONNECT_DATA", nStrLength) == 0)
#endif
							{
								nIndex++;
								i = i+nStrLength;
							} 
							else
							{
								return FALSE;
							}
						}
						else
						{
							return FALSE;
						}
					}
					else if (nLeft == nIndex && nRight == 5 && nEqualSign == nIndex)		//ADDRESS
					{
						nStrLength = 7;
						lpBuffer = arrayLine+i;
						if (i + nStrLength < nLineLength)
						{
#ifdef WIN32
							if (_strnicmp(lpBuffer, "ADDRESS", nStrLength) == 0)
#else
							if (strncasecmp(lpBuffer, "ADDRESS", nStrLength) == 0)
#endif
							{
								nIndex = 3;
								nLeft = 3;
								nRight = 0;
								nEqualSign = 3;
								j++;
								memset(&stAddr, 0, sizeof(stAddr));
								st.m_vecAddress.push_back(stAddr);
							}
							else
							{
								return FALSE;
							}
						}
						else
						{
							return FALSE;
						}
					}
					else
					{
						return FALSE;
					}
				}
				else if (9 == nIndex)			//DATABASENAME
				{
					if (bValue == FALSE)
					{
						if (nLeft == nIndex && nRight == 6 && nEqualSign == nIndex)
						{
							nStrLength = 12;
							lpBuffer = arrayLine+i;
							if (i + nStrLength < nLineLength)
							{
#ifdef WIN32
								if (_strnicmp(lpBuffer, "DATABASENAME", nStrLength) == 0)
#else
								if (strncasecmp(lpBuffer, "DATABASENAME", nStrLength) == 0)
#endif
								{
									i = i+nStrLength;
									bValue = TRUE;
								} 
								else
								{
									return FALSE;
								}
							}
							else
							{
								return FALSE;
							}
						}
						else
						{
							return FALSE;
						}
					} 
					else
					{
						if (nLeft == nIndex && nRight == 6 && nEqualSign == nIndex+1)
						{
							nStrLength = 0;
							lpBuffer = arrayLine+i;
							while (i < nLineLength)
							{
								if (IsCharacter(arrayLine[i]))
								{
									i++;
									nStrLength++;
									continue;
								}
								memcpy(st.m_cDatabaseName, lpBuffer, nStrLength);
								nIndex++;
								bValue = FALSE;
								break;
							}
							if (nIndex != 10)
							{
								return FALSE;
							}
						} 
						else
						{
							return FALSE;
						}
					}
				}
			}
			Sleep(1);
		}
		else
		{
			break;
		}
	}
	return FALSE;
}

BOOL CMemDBInterface::ParseConnectConfig(char *lpServerName, STRUCT_CONNECT_CONFIG &stConfig)
{
	FILE *fp;
	BOOL bRet;
	char lpWorkingPath[MAX_PATH], lpConfigPath[MAX_PATH];


	fp = NULL;
	GetModuleFileNameA(g_hVernoxInstance, lpWorkingPath, MAX_PATH);
	(strrchr(lpWorkingPath, '\\'))[0] = 0;
	(strrchr(lpWorkingPath, '\\'))[1] = 0;
	sprintf(lpConfigPath, "%sConfig\\Vernox.ini", lpWorkingPath);
	if(_access(lpConfigPath, 04) == -1)
	{
		//ļ
		GetModuleFileNameA(g_hVernoxInstance, lpWorkingPath, MAX_PATH);
		(strrchr(lpWorkingPath, '\\'))[1] = 0;
		sprintf(lpConfigPath, "%sVernox.ini", lpWorkingPath);
	}

	fp = fopen(lpConfigPath, "r");
	if (fp == NULL)
	{
		return FALSE;
	}
	bRet = ParseConnectConfig(lpServerName, fp, stConfig);
	fclose(fp);
	return bRet;
}

HRESULT CMemDBInterface::OpenCore(char * lpszServerName, char * lpszUserName, char * lpszPassword)
{
	int i, nLen;
	STRUCT_CONNECT_CONFIG stConfig;
	memset(stConfig.m_cSobeyDB, 0, sizeof(stConfig.m_cSobeyDB));
	memset(stConfig.m_cDatabaseName, 0, sizeof(stConfig.m_cDatabaseName));
	if(!ParseConnectConfig(lpszServerName, stConfig))
	{
		return MF_FAILED;
	}
	if(stConfig.m_vecAddress.size() == 0)
	{
		return MF_FAILED;
	}
	for(i = 0;i < stConfig.m_vecAddress.size();i++)
	{
		//TODO:	֧
		m_uServiceTCPIP		= inet_addr(stConfig.m_vecAddress[i].m_cHost);
		m_uServiceTCPPort	= htons((USHORT)atol(stConfig.m_vecAddress[i].m_cPort));
		if (lpszUserName != NULL)
		{
			nLen = strlen(lpszUserName);
			if (nLen > 23)
			{
				nLen = 23;
			}
			memset(m_pUserName, 0, sizeof(m_pUserName));
			memcpy(m_pUserName, lpszUserName, nLen);
		}
		if (lpszPassword != NULL)
		{
			nLen = strlen(lpszPassword);
			if (nLen > 23)
			{
				nLen = 23;
			}
			memset(m_pPassword, 0, sizeof(m_pPassword));
			memcpy(m_pPassword, lpszPassword, nLen);
		}
		if (MF_OK == OpenDatabase(FALSE))
		{
			m_bOpenFlag = TRUE;
			return MF_OK;
		}
	}
	return MF_FAILED;
}

HRESULT CMemDBInterface::UserLogOut(int& nRetValue)
{
	BYTE bToken;
	int i, nRet, nSize, nCount;
	MF_SERVICE_TCP_COMMAND stTcpCmd;
	CCriticalSectionPtr cs(&m_csInterFace);

	nCount = 0;
UserLogOut_Retry:
	bToken = CreateRandomFactor();
	nSize = sizeof(MF_SERVICE_TCP_COMMAND);
	memset(&stTcpCmd, 0, nSize);

	stTcpCmd.m_bVersion		= 1;
	stTcpCmd.m_bToken		= bToken;
	stTcpCmd.m_bType		= MF_INTERFACE_COMMAND_USERLOGOUT;
	memcpy(stTcpCmd.m_lpszParam, m_pUserName, 24);
	memcpy(stTcpCmd.m_lpszParam+24, m_pPassword, 24);

	nRet = send(m_skSocket, (char *)&stTcpCmd , nSize, 0);
	if(nRet  != nSize)
	{
		Trace(_T("UserLogout"), 0, 888042001, _T("ͨTCP˷ʧܣֵ%d룺%d"), nRet, WSAGetLastError());
		return E_FAIL;
	}
	//շ˷
	i = 0;
	while(TRUE)
	{
		memset(&stTcpCmd, 0, sizeof(MF_SERVICE_TCP_COMMAND));
		nRet = recv(m_skSocket, (char *)&stTcpCmd, sizeof(MF_SERVICE_TCP_COMMAND), 0);
		if(nRet == 0)
		{
			//ӱر
			Trace(_T("UserLogout"), 0, 888042002, _T("շʱӱرյĴ󣬷ֵ%d룺%d"), nRet, WSAGetLastError());
			return E_FAIL;
		}
		else if (nRet == SOCKET_ERROR)
		{
			//յSocket
			Trace(_T("UserLogout"), 0, 888042003, _T("շʱSocket󣬷ֵ%d룺%d"), nRet, WSAGetLastError());
			return E_FAIL;
		}
		else if(nRet == sizeof(MF_SERVICE_TCP_COMMAND))
		{
			//жϷصǷȷ
			if(stTcpCmd.m_bType == MF_INTERFACE_TCPCOMMAND_TEST)
			{
				continue;
			}
			else if(stTcpCmd.m_bToken != bToken)
			{
				nCount++;
				if(nCount > 3)
				{
					return E_FAIL;
				}

				Sleep(1);
				ClearSocket();
				goto UserLogOut_Retry;
			}
			else
			{
				break;
			}
		}
		else
		{
			Trace(_T("UserLogout"), 0, 888042004, _T("շʱֵģпǷ˵ȴʱͻԶTCPݣֵ%d룺%d"), nRet, WSAGetLastError());
			return E_FAIL;
		}
	}

	//жϷ
	nRetValue = stTcpCmd.m_lDigitalData;
	if(stTcpCmd.m_bType == MF_INTERFACE_TCPCOMMAND_FAIL)
	{
		//Ϊʧܽ
		Trace(_T("UserLogout"), 0, 888042010, _T("û¼עû%s"), m_pUserName);
		return E_FAIL;
	}
	else if(stTcpCmd.m_bType == MF_INTERFACE_TCPCOMMAND_OK)
	{
		memset(m_pUserName, 0, 24);
		memset(m_pPassword, 0, 24);
		return S_OK;
	}
	else
	{
		//ķֵ
		ClearSocket();
		Trace(_T("UserLogout"), 0, 888042011, _T("˷صͲȷͻԶTCPݣյͣ%d"), stTcpCmd.m_bType);
		return E_FAIL;
	}
	return E_FAIL;
}
